## Structure of the RAPS21 folder

In this folder, Singularity definition files and auxiliary files to build a containerized  instance of RAPS21 for both ***x86*** and ***Cray*** base images is provided.

The version of RAPS used for both containerization corresponds to the `raps21_27032024` bundle present at the `ecbench` ECMWF ftp endpoint. To that, a patch provided by ECMWF (thanks Balthasar!) has to be applied to integrate the `ifs-tests` suite to the bundle. These files cannot be uploaded to this repo due to license issues, please contact us if you need more details. We will assume the patched bundle to be present at `files/raps21-27032024_wtests.tar.gz`

The `definitions` folder contains Singularity Definition Files for building the target model either on x86 or Cray, respectively: `definition_file_raps_hpe2403.def` and `raps21_x86_singularity.def`. 

The base image used for x86 and Cray are direclty taken from the [NVIDIA HPC SDK catalog](https://catalog.ngc.nvidia.com/orgs/nvidia/containers/nvhpc) and [HPE Support Centre](https://support.hpe.com/connect) catalogues, as explained in the `README.md` in the main folder of this repo. In particular, as `RAPS21` is a CPU-enabled only model, no `ROCm` had to be added to the base container image for Cray.

We provide `env.sh` files which can be used to launch a RAPS build for either x86 or Cray container images. They can be found in the `files/arch/container` folder. The `files` folder also contains the needed `spack.yaml` files for the builds, respectively at `files/x86` and `files/cray`. Spack recipes are also included.

The build of the RAPS21 x86 and Cray container images can be launched respectively via

    singularity build raps_21_tests_x86.sif defintions/raps21_x86_singularity.def

and

    singularity build raps_21_tests_cray.sif defintions/raps21_cray_singularity.def

# Launching ifs-tests using a Container Image

When building RAPS, equipped with the ifs-tests suite, an `ifs-tests` folder is created in the `build` directory, from which base tests can be launched by a simple `ctest` command. Their execution is orchestrated by binaries located in the `ifs-tests/bin` subfolder. To launch these tests using a containerized instance of the `ifsMASTER.SP` executable we propose the following solution:

+ Start from a host-based build containing the ifs-tests suite,
+ Create a configuration file setting up the environment for container execution (path to container image, path to containerized binaries, bindings, flags, variables) and assign the absolute path of the configuration file to an environment variable e.g. 


        export CONTAINER_ENV=$(realpath set_container_tests.sh) 
    
    Examples of such configuration files for the hereby described container images can be found at `ifs-tests/set_container_tests.sh_x86` and  `ifs-tests/set_container_tests.sh_lumi`.

+ Replace the default `ifs-tests/bin/ifs-run` script with the one provided in this repo at `ifs-tests/bin/ifs-run_new`; for reference, the original script is also provided at `ifs-tests/bin/ifs-run_original`.

The `ifs-tests/bin/ifs-run_new` script adds a barrier which checks whether the `$CONTAINER_ENV` variables is set. If it is, the scripts read necessary variables from the configuration file and modifies on the fly the `$LAUNCH` command to allow execution from a container image. Below, a trace of the proposed `$LAUNCH` command for containers; see the `ifs-tests/bin/ifs-run_new` file.

+ Standard LAUNCH command  

        LAUNCH="${LAUNCHER} ${LAUNCHER_NPROC_FLAG} ${NPROC} ${LAUNCHER_NTHREAD_FLAG} ${NTHREAD} ${LAUNCHER_MEM_FLAG}  

+ Concatenate container setup  
        
        LAUNCH+="${CONTAINER_SW} ${CONTAINER_FLAGS} ${CONTAINER_GPU_FLAGS} ${CONTAINER_IMAGE}" 

+ Set the IFS/OSM executable to the containerized one 
    	
        IFS_EXECUTABLE=${CONTAINER_BINARIES} $(basename "$IFS_EXECUTABLE") 

+ Launch the simulation 
    	
        $LAUNCH $IFS_EXECUTABLE $IFS_ARGS 

In this way, it is also possible to perform a bitwise comparison between the host-based build holding the ***ifs-tests*** suite and the containerized one. To this end, one should
+ allocate necessary resources in the hosting cluster,
+ load the necessary `env.sh` file for the host-based build,
+ `export DR_HOOK_TRAPFPE=0`, otherwise giving errors in our RAPS version,
+ `export IFS_TEST_BITIDENTICAL=init`, so that results of ctests will be saved for reference,
+ launch the tests for the host-based build: `ctest`,
+ set the `CONTAINER_ENV` variable to switch to a container based executuion, `export CONTAINER_ENV=$(realpath set_container_tests.sh)`,
+ load the Singularity/Apptainer module,
+ `export IFS_TEST_BITIDENTICAL=check`, bitwise comparions against the reference values obtained from the host-based build.


## Launching RAPS using a container image

The container images were tested on both x86 and Cray clusters (e.g. *LEONARDO* and *LUMI*) also for full model runs. To this end, one does not strictly need a host-based build on which to lean on. Still, a series of RAPS-related files needs to be binded at runtime from the host, namely:
+ `/raps/bin`: provide binaries executed at runtime, 
+ `/raps/eccodes/definitions`: needed to be read at runtime, 
+ `/raps/eccodes/ifs_samples`: as above, 
+ `/raps/etc/fdb`: containing configuration files for fdb output, 
+ `/raps/inputs`: raps inputs files. 

Then, a modification similar to the one described in the previous paragraph can be applied to the `model (hres)` script to modify on the fly the `$LAUNCHCMD` command, allowing for execution from containerized instances.

For the purpose of simple testing, one can manually modify the `$LAUNCHCMD` variable in the `model (hres)` script, similar to the following:

    if [[ -z "${LAUNCHCMD:-}" ]] ; then
        
        # Original LAUNCHCMD
        # LAUNCHCMD="$launcher ${wrapper:-} ${perf:-} $exe $args"
        
        # set container path and binary
        cont="/path/to/container/raps_21_tests.sif"
        exe="/raps/build/bin/ifsMASTER.SP"

        # bind the rundir for runtime writing
        export SINGULARITY_BIND=${rundir},$SINGULARITY_BIND

        # The refactored launch command
        LAUNCHCMD="$launcher ${wrapper:-} ${perf:-} singularity exec $cont $exe $args"
    fi

The proposed modifications for `ifs-tests` would apply similarly in this case. Alternatively, implementation using `--flags` is surely possible following the same logic.
