## Description of the repository

This repo serves as a support for the Deliverable ***"DE_360_D360.2.3.1_202503: Final cloud deployment containerised solution"*** in the framework of Work Package 2 of Destination Earth 360. In this repository we provide Singualrity definition files and other needed auxiliary files for the build of the ecWAM, ecTrans and RAPS models as described in the relative report. In each subfolder, a `README.md` with a more detailed description of the files therein included is provided.

+ The `ecwam` folder just mirrors the specific [ecWAM-container repository](https://gitlab.hpc.cineca.it/de360/ecwam-container). A `README.md` is provided, detailing the structure of the repository.

+ The `ecTrans` folder contains the Singularity definition file used for building a containerized version of the ecTrans model on LUMI. Auxiliary files such as Spack recipes are also provided, along with an example script for the execution.

+ The `RAPS21` folder contains Singularity definition files used for building a containerized version of RAPS21 equipped with the ***ifs-tests suite***. Auxiliary files and discussion on how to run the containerized RAPS are provided in the relative `README.md` file. 

### Base images for x86 and Cray container images

Base images for x86 containers are taken from the [NVIDIA HPC SDK catalog](https://catalog.ngc.nvidia.com/orgs/nvidia/containers/nvhpc) and can be directly fetched via e.g.
   
    singularity build nvhpc-23-1.sif docker://nvcr.io/nvidia/nvhpc:23.1-devel-cuda12.0-ubuntu22.04 

For Cray containers, base images not equipped with `ROCm` can be fetched direclty from the [HPE Support Centre](https://support.hpe.com/connect). There, an URL is provided from which a `.tar.gz` file tagged `HPE_CPE_Container_XY.ZW.tar` can be downloaded, holding a *Docker Image Archive* for the specific HPE CPE base image. In order to use it as base image for a Singularity build, it should first be converted in an *OCI Image Archive*, hence in a *Singularity Image Format* using either *gunzip* or *skopeo*, for example:

    curl -o HPE_CPE_Container_24.03.tar.gz   $URL 
    tar -xzf HPE_CPE_Container_24.03.tar.gz  
    gunzip HPE_CPE_Container_24.03/cpe_2403.tgz
    singularity build ccpe_2403.sif docker-archive:// HPE_CPE_Container_24.03/cpe_2403.tar 
    # skopeo can be used as an alternative to gunzip 
    # skopeo copy docker-archive:cpe_2403.tgz oci-archive:cpe_2403_oci.tar 
    # singularity build ccpe_2403.sif oci-archive:cpe_2403_tar.oci 

Base images provided with a `ROCm` installation, such as the one used for the containerization of ecTrans, were provided to the project by [CSC personnel]( https://research.csc.fi/high-performance-computing/). They are built starting from the *same* base images downloadable from the HPE Support Center, with the addition of the ROCm module made directly from the LUMI repository.
