## Containerization of the ecTrans model

In this folder, we provide guidance on building a containerized instance of the [ecTrans model](https://github.com/ecmwf-ifs/ectrans), precisely the ***redgreengpu*** branch, on the [LUMI supercomputer]( https://docs.lumi-supercomputer.eu/ ). The Singularity definition file to build is provided at `definitions/ectrans_singularity.def`

The base image used for the build is tagged `ccpe-24.03-rocm-6.0.sif`. This base image was obtained starting from the `HPE_CPE_Container_24.03.tar` Docker Image Archive provided by the HPE Support Center, which can be obtained as described in the `README.md` in the base folder of this repository. As ecTrans can be compiled with GPU support, a `ROCm` instance is also needed inside the container image to build and run GPU supported code on LUMI. In this case, the `ROCm/6.0.3` package was added to the container image directly from the LUMI repository to the HPE base image. Please contact CSC support to get access to such an image.

In `/files`, files needed for the build are listed. They include
   - `files/spack.yaml` a Spack recipe to install a few packages needed by ecTrans but missing in the CPE software stack, in this case *cmake* only.
   - `files/toolchain.cmake`: needed modifications (in a host-based build as well) for correctly compiling the model.

Finally, the `ectrans_job.sh` provides a blueprint of a SLURM script used for testing the containerized model on LUMI. The script iincludes also the load of the `singularity-bindings/24.03` module, which can be installed in LUMI following the [documentation page](https://docs.lumi-supercomputer.eu/software/installing/easybuild/#a-special-case-modules-for-singularity-containers).

Upon getting the correct base image, the build can be launched via `Singularity` on a machine where the user has either `sudo`, `proot` or `fakeroot` privileges via

    singularity build ectrans_container.sif defintions/ectrans_singularity.def 