#!/bin/bash -l
#SBATCH --job-name=cont_ectransTests   	# Job name
#SBATCH --output=ectransTest.%j.log	    # Name of stdout output file
#SBATCH --error=ectransTest.%j.log  	# Name of stderr error file
#SBATCH --partition=standard-g 	 	    # partition name
#SBATCH --nodes=16                      # NUmber of nodes used
#SBATCH --ntasks-per-node=8     	    # 8 MPI ranks per node, 16 total (2x8)
#SBATCH --gpus-per-node=8       	    # Allocate one gpu per MPI rank
#SBATCH --time=02:00:00       		    # Run time (d-hh:mm:ss)
#SBATCH --account=project_465000527  	# Project for billing


echo "Grepping for SLURM environment..."
env | grep SLURM

module load LUMI && module load partition/C && module load systools
module load singularity-bindings/24.03
module load CrayEnv PrgEnv-cray cce/17.0.1 craype-accel-amd-gfx90a rocm/6.0.3 cray-fftw buildtools
module list

# Bindnings. Some bindings are also implicitly added via the singularity-bindings module
export SINGULARITY_BIND=/opt/cray/libfabric,/var/spool,/etc/host.conf,/etc/hosts,/etc/nsswitch.conf,/etc/resolv.conf,/etc/ssl/openssl.cnf,/usr/lib64/libcxi.so.1,/run/cxi,/opt/rocm-6.0.3/lib/libhsa-runtime64.so
export SINGULARITYENV_LD_LIBRARY_PATH=/opt/cray/libfabric/1.15.2.0/lib64

# Script to allow for GPU execution on LUMI
cat << EOF > select_gpu
#!/bin/bash
export ROCR_VISIBLE_DEVICES=\$SLURM_LOCALID
exec \$*
EOF
chmod +x ./select_gpu

CPU_BIND="map_cpu:49,57,17,25,1,9,33,41"

# Options for GPU offloading on LUMI
export MPICH_GPU_SUPPORT_ENABLED=1  # Allows for GPU MPI aware worklfows
export MPICH_SMP_SINGLE_COPY_MODE=NONE
export MPICH_GPU_IPC_ENABLED=0
export FI_CXI_RX_MATCH_MODE=software

container="ectrans_container.sif"
exe="/ectrans_bundle/ectrans/build/bin/ectrans-benchmark-gpu-sp"
resolution=450

srun --ntasks-per-node=$SLURM_TASKS_PER_NODE --hint=nomultithread -N $SLURM_NNODES -n $SLURM_NTASKS --unbuffered --cpu-bind=${CPU_BIND} ./select_gpu singularity exec --rocm $container $exe -t $resolution --nlev 10 --norms
rm -rf ./select_gpu
