## General informations

In this repository we provide all the needed files and instructions to build a singularity container image for running the ECMWF Ocean Wave Model, aka the *ecWAM* model. The container image supports an hybrid level of parallelism `MPI+OpenMP` as well as GPU support via `OpenACC`. Currently it has been succesfully tested in the CINECA HPC clusters *LEONARDO* and *GALILEO100*.


### Structure of the repo

The directories in the repo are so divided:
+ `/bind_folders`: some empty folders to be used as mounting point to allow writing at runtime 
+ `/build_files`: all the files needed in the built of the container image
+ `/configuration_files`: scripts to set up the environment for running the containerized model. Currently on: *LEONARDO*, *GALILEO100*, locally
+ `/definitions`: templates for the singularity definition files used to build the container image
+ `/ecwam_scripts`: scripts to run the ecwam-model
+ `/tests`: a minimal set of tests. As a demonstration we just run `/tests/etopo1_oper_an_fc_O48.yml`.

Please have a look at the `README.md` files in the various folders for more detailed informations.


### Building the container image

First off, you will need a working version of `Singularity` installed in your workstation (see e.g. [here](https://docs.sylabs.io/guides/latest/user-guide/quick_start.html)), we used the version `3.9.9`. 

In order to create the *default* container image, just run the `create_container.sh` script, 

    [/ecwam-container] $ ./create_container.sh

you will need to give sudo rights. The script templates the `definitions/template.def` file with the current path of the repository and the default values for the `nvhpc` and `cuda` versions, respectively `23.1` and `11.8`. Hence, it generates a container in a *Singularity Image Format* (sif) at `images/container_ecwam.sif`. The build can take up to one hour, depending on the hardware features of the workstation used, and generates a log file as `output_build.log`. 

One can also create a container in a `sandbox` format just executing the script with the added `sandbox` flag as in

    [/ecwam-container] $ ./create_container.sh --sandbox

This can be useful for development, but occupies fairly more space (up to double) with respect to the *.sif* file.



### Running the container image

From the above procedure, the `container_ecwam.sif` image file has been built. One can run a *test of the ecWAM-model* just executing the `run_script.sh`, optionally specifying the path to the container image (or the default `/ecwam-container/images/container_ecwam.sif` is assumed)

    [/ecwam-container] $ ./run_script.sh    # optional [/path/to/my/ecwam/sif]

The `run_script.sh` first sources the `configuration_files/setup-env-local.sh` file, which performs the following operations:
+ sets the `$PREPROC`, `$PRESET`, `$MODEL` and `$CHIEF` variables, needed to run the `ecwam_scripts`
+ sets the `$ECWAM_RUN_DIR`, `$ECWAM_DATA_DIR` and `$ECWAM_CACHE_PATH` environment variables for runtime writing
+ sets the `$SINGULARITY_BIND` environment variable, specifying binds to be performed whence an instance of the container is started

Thus, it runs in sequence the `ecwam_run_preproc`, `ecwam_run_preset` and `ecwam_run_model` binaries on the `/tests/etopo1_oper_an_fc_O48.yml` test case. The execution will be handled by the *openmpi internal to the container*. To run in a cluster in a Singularity hybrid mode, have a look to the `configuration_files` subdirectory and the `README.md` therein.



### Current issues
There are a list of issues we are carrying along for the moment. We list them here in a loosely ordered manner of importance:

1. Spack can have a somewhat unpredictable behaviour in a container environment. For example, during the installation phase, some of the environment variables are dropped. The temporary solution is to `spack install` with the flag `dirty` (see the `/opt/spack/etc/spack/config.yaml` file in the container, or equivalently in the repo: `build_files/spack_yaml/config.yaml`). The `dirty` flag allows spack to keep all the environment variables from the container. As the name suggests, using the `dirty` flag is usually considered bad practice, *however* we start from a virgin container image, on which we have control of the environment
30. When trying to compile with the `multio` package an `undefined reference` error occurs. This is one of the *ECMWF* packages, not necessary to run the model but still suggested. The current fix is to just drop said package, manually replacing the `/ecwam_bundle/ecwam/ecwam/CMakeLists.txt` and `/ecwam_bundle/ecwam/ecwam_naan-phys-gpu/CMakeLists.txt` files, see also the `README.md` file in `build_files/`
40. In building the container, we perform various `apt-get install` as well as `apt-get update`. Some of the packages are then provided as *externals* to spack. In order to provide a stable build, at some point we will have to fix freeze the versions of all the downloaded packages 