Introduction
============
A Fortran library purpose-built for IFS data-structures that facilitates the creation and management (including GPU/accelerator offload) of field objects in scientific code. GPU offload capabilities are currently only supported for Nvidia GPUs.

## Requirements:
To build FIELD_API you need:
- A Fortran 2008 compliant compiler
- CMake version 3.20 or newer
- [ecbuild](https://github.com/ecmwf/ecbuild) version 3.7 or newer
- The [fypp](https://github.com/aradi/fypp) preprocessor (used as a meta-programming tool)
- [fiat](https://github.com/ecmwf-ifs/fiat) (optional)

## Instructions to build and test
If using fiat outside of a bundle install, ensure the following environment variable is set:
```bash
export fiat_ROOT=<path-to-fiat>
```

Configure the build:
```bash
ecbuild -B build
```

To build FIELD_API:
```bash
cmake --build build
```

To test FIELD_API:
```bash
ctest --test-dir build/tests
```

## Architectures
The `arch` directory contains scripts to configure the build environment on frequently used hosts. For example, to build using the Nvidia compiler suite on the ECWF Atos cluster, run:
```bash
source arch/ecmwf/hpc2020/nvhpc/22.1/env.sh
```
The `env.sh` scripts also load CMake toolchaines that contain appropriate compilers flags for each architecture.

## Features
Features of FIELD_API can be toggled by passing the following argument to the configure step: `-DENABLE_FIELD_API_FEATURE=ON/OFF`. The table below lists all the available features:

| Feature | Default | Description |
|:--- |:--- |:--- |
| TESTS | ON | Build the FIELD_API testing suite. |
| FIAT_BUILD | ON | Import the Fortran utility modules (oml_mod, parkind1 and abor1) from fiat. If this option is disabled, then the path to the directory containing `oml_mod.F90`, `abor1.F90` and `parkind1.F90` must be specified using the CMake variable `FIELD_API_UTIL_MODULE_PATH`. |
| CUDA | ON | Enable the use of CUDA Fortran in FIELD_API. |

## Precision
By default, FIELD_API builds using fiat, and links to parkind_dp and parkind_sp. As a result, both double and single precision static libraries are built i.e. field_api_dp and field_api_sp. If however FIAT_BUILD is disabled, then only a single static library field_api is produced. The precision in this case is set by the contents of `${FIELD_API_UTIL_MODULE_PATH}/parkind1.F90`. 
