! (C) Copyright 1989- ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!

      MODULE YOWTABL

      USE PARKIND_WAVE, ONLY : JWIM, JWRB, JWRU

      IMPLICIT NONE

!*    ** *TABLE* - TABLE FOR TOTAL STRESS AND HIGH FREQ STRESS. !!!! (obsolete !!!
!                  AND TABLES FOR THE CORRECTION OF ALTIMETER HS.
!*    ** *TABLE* - TABLE FOR 2ND-ORDER INTERACTION COEFFICIENTS.

      INTEGER(KIND=JWIM), PARAMETER :: ITAUMAX=400 
      INTEGER(KIND=JWIM), PARAMETER :: JUMAX=300 
      INTEGER(KIND=JWIM), PARAMETER :: IUSTAR=500 
      INTEGER(KIND=JWIM), PARAMETER :: IALPHA=400 
      INTEGER(KIND=JWIM), PARAMETER :: ILEVTAIL=50
      INTEGER(KIND=JWIM), PARAMETER :: IAB=200
      INTEGER(KIND=JWIM), PARAMETER :: NFREHF=49 

      INTEGER(KIND=JWIM) :: MR 
      INTEGER(KIND=JWIM) :: MA 
      INTEGER(KIND=JWIM) :: NFREH 
      INTEGER(KIND=JWIM) :: NANGH 
      INTEGER(KIND=JWIM) :: NMAX

      REAL(KIND=JWRB), PARAMETER   :: EPS1 = 0.00001_JWRB
      REAL(KIND=JWRB), PARAMETER   :: UMAX = 75.0_JWRB
      REAL(KIND=JWRB), ALLOCATABLE :: TAUT(:,:,:) 
      REAL(KIND=JWRB)              :: DELTAUW 
      REAL(KIND=JWRB)              :: DELU 
      REAL(KIND=JWRB), PARAMETER   :: USTARM = 5.0_JWRB
      REAL(KIND=JWRB), PARAMETER   :: ALPHAMCOEF = 40.0_JWRB
      REAL(KIND=JWRB), PARAMETER   :: XLEVTAILM = 0.02_JWRB
      REAL(KIND=JWRB), ALLOCATABLE :: TAUHFT(:,:,:) 
      REAL(KIND=JWRB), ALLOCATABLE :: PHIHFT(:,:,:) 
      REAL(KIND=JWRB), ALLOCATABLE :: TAUHFT2(:,:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: PHIHFT2(:,:,:)
      REAL(KIND=JWRB)              :: DELUST 
      REAL(KIND=JWRB)              :: DELALP 
      REAL(KIND=JWRB)              :: DELTAIL
      REAL(KIND=JWRB), ALLOCATABLE :: FAC0(:,:,:,:) 
      REAL(KIND=JWRB), ALLOCATABLE :: FAC1(:,:,:,:) 
      REAL(KIND=JWRB), ALLOCATABLE :: FAC2(:,:,:,:) 
      REAL(KIND=JWRB), ALLOCATABLE :: FAC3(:,:,:,:) 
      REAL(KIND=JWRB), ALLOCATABLE :: FAK(:) 
      REAL(KIND=JWRB), ALLOCATABLE :: FRHF(:) 
      REAL(KIND=JWRB), ALLOCATABLE :: DFIMHF(:) 
      REAL(KIND=JWRB)              :: SWELLFT(IAB)
      REAL(KIND=JWRB)              :: XMR
      REAL(KIND=JWRB)              :: XMA
      REAL(KIND=JWRB)              :: DELTHH

      INTEGER(KIND=JWIM),ALLOCATABLE :: IM_P(:,:)
      INTEGER(KIND=JWIM),ALLOCATABLE :: IM_M(:,:)
                 
      REAL(KIND=JWRB), ALLOCATABLE :: OMEGA(:) 
      REAL(KIND=JWRB), ALLOCATABLE :: DFDTH(:) 
      REAL(KIND=JWRB), ALLOCATABLE :: THH(:) 
      REAL(KIND=JWRB), ALLOCATABLE :: TA(:,:,:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: TB(:,:,:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: TC_QL(:,:,:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: TT_4M(:,:,:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: TT_4P(:,:,:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: TFAKH(:,:)

!*     VARIABLE.   TYPE.     PURPOSE.
!      ---------   -------   --------
!
!***   STRESS AND HIGH-FREQUENCY WAVE STRESS.
!      -------------------------------------
!  
!      *ITAUMAX*   INTEGER   TABLE DIMENSION.
!      *JUMAX*     INTEGER   TABLE DIMENSION.
!      *IUSTAR*    INTEGER   TABLE DIMENSION.
!      *IALPHA*    INTEGER   TABLE DIMENSION.
!      *ILEVTAIL*  INTEGER   TABLE DIMENSION.
!      *IAB*       INTEGER   TABLE DIMENSION.
!      *NFREHF*    INTEGER   EXTENDED NUMBER OF FREQUENCIES.
!      *EPS1*      REAL      SMALL MULTIPLICATIVE NUMBER TO MAKE SURE THAT A SOLUTION
!                            IS OBTAINED IN ITERATION WITH TAU>TAUW.
!      *UMAX*      REAL      MAXIMUM WIND SPEED IN STRESS TABLE.
!      *TAUT*      REAL      STRESS TABLE.
!      *DELTAUW*   REAL      WAVE STRESS INCREMENT.
!      *DELU*      REAL      WIND INCREMENT.
!      *USTARM*    REAL      MAXIMUM FRICTION VELOCITY IN STRESS TABLE.
!      *ALPHAMCOEF*REAL      MULTIPLICATIVE FACTOR TO SET THE MAXIMUM CHARNOCK
!                            BASED ON THE MINIMUM CHARNOCK VALUE IN STRESS TABLE.
!      *XLEVTAILM* REAL      MAXIMUM TAIL FACTOR (SATURATION LEVEL) IN STRESS TABLE.
!      *TAUHFT*    REAL      HIGH FREQUENCY STRESS TABLE for ECMWF PHYSICS.
!      *TAUHFT2*   REAL      HIGH FREQUENCY STRESS TABLE for METEO FRANCE PHYSICS.
!      *PHIHFT*    REAL      HIGH FREQUENCY WIND INPUT WAVE ENERGY FLUX TABLE for ECMWF PHYSICS.
!      *PHIHFT2*   REAL      HIGH FREQUENCY WIND INPUT WAVE ENERGY FLUX TABLE for METEO FRANCE PHYSICS.
!      *DELUST*    REAL      USTAR INCREMENT.
!      *DELALP*    REAL      ALPHA INCREMENT.
!      *DELTAIL*    REAL     ALPHA INCREMENT.
!
!***   CORRECTIONS TO ALTIMETER.
!      ------------------------
!
!      *FAC0*      REAL      TABLE FOR HIGHER MOMENT CALCULATION. 
!      *FAC1*      REAL      TABLE FOR HIGHER MOMENT CALCULATION. 
!      *FAC2*      REAL      TABLE FOR HIGHER MOMENT CALCULATION. 
!      *FAC3*      REAL      TABLE FOR HIGHER MOMENT CALCULATION. 
!      *FAK*       REAL      WAVE NUMBERS FOR HIGHER MOMENT CALCULATION.
!      *FRHF*      REAL      FREQUENCIES FOR HIGHER MOMENT CALCULATION.
!      *DFIMHF*    REAL      FREQUENCY DIRECTION AREA FOR HIGHER
!                            MOMENT CALCULATION.
!      *SWELLFT*   REAL      FRICTION COEFFICIENTS IN OSCILLATORY BOUNDARY LAYERS.
!
!***   SECOND AND THIRD-ORDER INETERACTION COEFFICIENTS.
!      ------------------------------------------------
!
!       MR        INTEGER   THINNING PARAMETER IN FREQUECY SPACE
!       MA        INTEGER   THINNING PARAMETER IN ANGULAR SPACE
!       NFREH     INTEGER   NUMBER OF FREQUENCIES
!       NANGH     INTEGER   NUMBER OF DIRECTIONS
!       NMAX      INTEGER   MAXIMUM OF FREQUENCY INDEX
!       XMR       REAL      INVERSE OF THINNING FACTOR IN FREQUENCY SPACE
!       XMA       REAL      INVERSE OF THINNING FACTOR IN ANGULAR SPACE
!       DELTHH    REAL      DIRECTIONAL INCREMENT IN REDUCED SPACE
!       OMEGA     REAL      ANGULAR FREQUENCY ARRAY
!       DFDTH     REAL      FREQUENCY DIRECTION INCREMENT IN REDUCED SPACE
!       THH       REAL      DIRECTION ARRAY
!       TA        REAL      TABLE FOR MINUS INTERACTIONS
!       TB        REAL      TABLE FOR PLUS INTERACTIONS
!       TC_QL     REAL      TABLE FOR QUASI-LINEAR INTERACTIONS
!       TT_4M     REAL      TABLE FOR STOKES FREQUENCY CORRECTION
!       TT_4P     REAL      TABLE FOR STOKES FREQUENCY CORRECTION
!       IM_P      INTEGER   TABLE FOR WAVENUMBER M2 PLUS
!       IM_M      INTEGER   TABLE FOR WAVENUMBER M2 MIN
!       TFAKH     REAL      WAVENUMBER TABLE IN REDUCED SPACE
!
! ----------------------------------------------------------------------
      END MODULE YOWTABL
