! (C) Copyright 1989- ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!

SUBROUTINE WAMINTGR_LOKI_GPU(CDTPRA, CDATE, CDATEWH, CDTIMP, CDTIMPNEXT,  &
 &                   BLK2GLO,                                     &
 &                   WVENVI, WVPRPT, FF_NOW, FF_NEXT, INTFLDS,    &
 &                   WAM2NEMO, MIJ, FL1, XLLWS, TIME1)


! ----------------------------------------------------------------------

!**** *WAMINTGR* - 3-G WAM MODEL - TIME INTEGRATION OF WAVE FIELDS.

!*    PURPOSE.
!     --------

!       COMPUTATION OF THE 2-D FREQUENCY-DIRECTION WAVE SPECTRUM AT ALL
!       GRID POINTS FOR A GIVEN INITIAL SPECTRUM AND FORCING SURFACE
!       STRESS FIELD.

!     REFERENCE.
!     ----------

!         IFS DOCUMENTATION, part VII 

! -------------------------------------------------------------------

USE PARKIND_WAVE, ONLY : JWIM, JWRB, JWRU
USE YOWDRVTYPE  , ONLY : WVGRIDGLO, ENVIRONMENT, FREQUENCY, FORCING_FIELDS,  &
 &                       INTGT_PARAM_FIELDS, WAVE2OCEAN

USE YOWCOUP  , ONLY : LWNEMOCOU, NEMONTAU
USE YOWGRID  , ONLY : NPROMA_WAM, NCHNK
USE YOWPARAM , ONLY : NIBLO, NANG, NFRE
USE YOWPCONS , ONLY : EPSMIN
USE YOWSTAT  , ONLY : CDTPRO, IDELPRO, IDELT, IDELWI, LLSOURCE
USE YOWWIND  , ONLY : CDAWIFL, CDATEWO, CDATEFL
USE YOWFIELD_MOD, ONLY : FREQUENCY_FIELD, ENVIRONMENT_FIELD, FORCING_FIELDS_FIELD, &
 &  WAVE2OCEAN_FIELD, INTGT_PARAM_FIELDS_FIELD, SOURCE_CONTRIBS_FIELD

USE YOMHOOK  , ONLY : LHOOK, DR_HOOK, JPHOOK

! ----------------------------------------------------------------------

IMPLICIT NONE

#include "implsch.intfb.h"
#include "incdate.intfb.h"
#include "newwind.intfb.h"
#include "propag_wam.intfb.h"
#include "wam_user_clock.intfb.h"

CHARACTER(LEN=14), INTENT(IN)                                            :: CDTPRA  ! DATE FOR CALL PROPAGATION
CHARACTER(LEN=14), INTENT(INOUT)                                         :: CDATE  ! CURRENT DATE
CHARACTER(LEN=14), INTENT(INOUT)                                         :: CDATEWH ! DATE OF THE NEXT FORCING FIELDS
CHARACTER(LEN=14), INTENT(INOUT)                                         :: CDTIMP  ! START DATE OF SOURCE FUNCTION INTEGRATION
CHARACTER(LEN=14), INTENT(INOUT)                                         :: CDTIMPNEXT  ! NEXT START DATE OF SOURCE FUNCTION INTEGRATION
TYPE(WVGRIDGLO), INTENT(IN)                                              :: BLK2GLO  ! BLOCK TO GRID TRANSFORMATION
TYPE(ENVIRONMENT), INTENT(INOUT)                                         :: WVENVI !  WAVE ENVIRONMENT FIELDS
TYPE(FREQUENCY), INTENT(INOUT)                                           :: WVPRPT  ! WAVE PROPERTIES FIELDS
TYPE(FORCING_FIELDS), INTENT(INOUT)                                      :: FF_NOW  ! FORCING FIELDS AT CURRENT TIME
TYPE(FORCING_FIELDS), INTENT(IN)                                         :: FF_NEXT  !  DATA STRUCTURE WITH THE NEXT FORCING FIELDS
TYPE(INTGT_PARAM_FIELDS), INTENT(INOUT)                                  :: INTFLDS  ! INTEGRATED/DERIVED PARAMETERS
TYPE(WAVE2OCEAN), INTENT(INOUT)                                          :: WAM2NEMO  ! WAVE FIELDS PASSED TO NEMO
INTEGER(KIND=JWIM), DIMENSION(NPROMA_WAM, NCHNK), INTENT(INOUT)          :: MIJ  ! LAST FREQUENCY INDEX OF THE PROGNOSTIC RANGE
REAL(KIND=JWRB), DIMENSION(NPROMA_WAM, NANG, NFRE, NCHNK), INTENT(INOUT) :: FL1
REAL(KIND=JWRB), DIMENSION(NPROMA_WAM, NANG, NFRE, NCHNK), INTENT(INOUT) :: XLLWS ! TOTAL WINDSEA MASK FROM INPUT SOURCE TERM

REAL(KIND=JWRB), INTENT(INOUT) :: TIME1(3)
REAL(KIND=JWRB) :: TIME0, TIME2


INTEGER(KIND=JWIM) :: IJ, K, M
INTEGER(KIND=JWIM) :: ICHNK
INTEGER(KIND=JWIM) :: IDELWH

! Objects to store fields
TYPE(FREQUENCY_FIELD) :: WVPRPT_FIELD
TYPE(ENVIRONMENT_FIELD) :: WVENVI_FIELD
TYPE(FORCING_FIELDS_FIELD) :: FF_NOW_FIELD
TYPE(WAVE2OCEAN_FIELD) :: WAM2NEMO_FIELD
TYPE(INTGT_PARAM_FIELDS_FIELD) :: INTFLDS_FIELD
TYPE(SOURCE_CONTRIBS_FIELD) :: SRC_CONTRIBS

! DEVICE POINTERS
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: FL1_DPTR(:,:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: XLLWS_DPTR(:,:,:,:) => NULL()
INTEGER(KIND=JWIM), POINTER, CONTIGUOUS :: MIJ_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: WAVNUM_DPTR(:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: CGROUP_DPTR(:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: CIWA_DPTR(:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: CINV_DPTR(:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: XK2CG_DPTR(:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: STOKFAC_DPTR(:,:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: EMAXDPT_DPTR(:,:) => NULL()
INTEGER(KIND=JWIM), POINTER, CONTIGUOUS :: INDEP_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: DEPTH_DPTR(:,:) => NULL()
INTEGER(KIND=JWIM), POINTER, CONTIGUOUS :: IOBND_DPTR(:,:) => NULL()
INTEGER(KIND=JWIM), POINTER, CONTIGUOUS :: IODP_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: CICOVER_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: WSWAVE_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: WDWAVE_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: AIRD_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: WSTAR_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: UFRIC_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUW_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUWDIR_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: Z0M_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: Z0B_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: CHRNCK_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: CITHICK_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOUSTOKES_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOVSTOKES_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOSTRN_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NPHIEPS_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NTAUOC_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NSWH_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NMWP_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOTAUX_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOTAUY_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOWSWAVE_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: NEMOPHIF_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: WSEMEAN_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: WSFMEAN_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: USTOKES_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: VSTOKES_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: STRNMS_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUXD_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUYD_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUOCXD_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUOCYD_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: TAUOC_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: PHIOCD_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: PHIEPS_DPTR(:,:) => NULL()
REAL(KIND=JWRB), POINTER, CONTIGUOUS :: PHIAW_DPTR(:,:) => NULL()

REAL(KIND=JPHOOK) :: ZHOOK_HANDLE


LOGICAL, SAVE :: LLNEWFILE

DATA LLNEWFILE / .FALSE. /


! ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WAMINTGR',0,ZHOOK_HANDLE)

!*     PROPAGATION TIME
!      ----------------

!!$acc enter data copyin(BLK2GLO, WVENVI, WVPRPT, FL1) copyout(FL1)

CALL SRC_CONTRIBS%INIT(FL1=FL1)
CALL SRC_CONTRIBS%UPDATE_DEVICE(FL1=FL1_DPTR)
!$acc data present(FL1_DPTR)

!!$acc data copyin(BLK2GLO, WVENVI, WVPRPT, FL1) copyout(FL1)
IF (CDATE == CDTPRA) THEN
  TIME0=-WAM_USER_CLOCK()

!!$acc data present(BLK2GLO, WVENVI, WVPRPT, FL1)
!!$acc data present(FL1)
  CALL PROPAG_WAM(BLK2GLO, WVENVI, WVPRPT, FL1_DPTR)
!!$acc end data 
  TIME1(1) = TIME1(1) + (TIME0+WAM_USER_CLOCK())*1.E-06
  CDATE = CDTPRO
ENDIF
!$acc end data

!!$acc exit data delete(BLK2GLO, WVENVI, WVPRPT)

!* RETRIEVING NEW FORCING FIELDS IF NEEDED.
!  ----------------------------------------
CALL NEWWIND(CDTIMP, CDATEWH, LLNEWFILE,      &
 &           WVPRPT, FF_NOW, FF_NEXT)

! IT IS TIME TO INTEGRATE THE SOURCE TERMS
! ----------------------------------------
IF (CDATE >= CDTIMPNEXT) THEN
! COMPUTE UPDATE DUE TO SOURCE TERMS
  CALL GSTATS(1431,0)
  IF (LLSOURCE) THEN

    TIME2=-WAM_USER_CLOCK()
    CALL WVPRPT_FIELD%INIT(WAVNUM=WVPRPT%WAVNUM, CGROUP=WVPRPT%CGROUP, CIWA=WVPRPT%CIWA, CINV=WVPRPT%CINV, XK2CG=WVPRPT%XK2CG, &
 &                         STOKFAC=WVPRPT%STOKFAC)
    CALL WVENVI_FIELD%INIT(EMAXDPT=WVENVI%EMAXDPT, INDEP=WVENVI%INDEP, DEPTH=WVENVI%DEPTH, IOBND=WVENVI%IOBND, IODP=WVENVI%IODP)
    CALL FF_NOW_FIELD%INIT(AIRD=FF_NOW%AIRD, WDWAVE=FF_NOW%WDWAVE, CICOVER=FF_NOW%CICOVER, WSWAVE=FF_NOW%WSWAVE, &
&                          WSTAR=FF_NOW%WSTAR, UFRIC=FF_NOW%UFRIC, TAUW=FF_NOW%TAUW, TAUWDIR=FF_NOW%TAUWDIR, &
&                          Z0M=FF_NOW%Z0M, Z0B=FF_NOW%Z0B, CHRNCK=FF_NOW%CHRNCK, CITHICK=FF_NOW%CITHICK)
    CALL WAM2NEMO_FIELD%INIT(NEMOUSTOKES=WAM2NEMO%NEMOUSTOKES, NEMOVSTOKES=WAM2NEMO%NEMOVSTOKES, NEMOSTRN=WAM2NEMO%NEMOSTRN, &
&                            NPHIEPS=WAM2NEMO%NPHIEPS, NTAUOC=WAM2NEMO%NTAUOC, NSWH=WAM2NEMO%NSWH, NMWP=WAM2NEMO%NMWP, &
&                            NEMOTAUX=WAM2NEMO%NEMOTAUX, NEMOTAUY=WAM2NEMO%NEMOTAUY, NEMOWSWAVE=WAM2NEMO%NEMOWSWAVE, &
&                            NEMOPHIF=WAM2NEMO%NEMOPHIF)
    CALL INTFLDS_FIELD%INIT(WSEMEAN=INTFLDS%WSEMEAN, WSFMEAN=INTFLDS%WSFMEAN, USTOKES=INTFLDS%USTOKES, VSTOKES=INTFLDS%VSTOKES, &
&                           STRNMS=INTFLDS%STRNMS, TAUXD=INTFLDS%TAUXD, TAUYD=INTFLDS%TAUYD, TAUOCXD=INTFLDS%TAUOCXD, &
&                           TAUOCYD=INTFLDS%TAUOCYD, TAUOC=INTFLDS%TAUOC, PHIOCD=INTFLDS%PHIOCD, PHIEPS=INTFLDS%PHIEPS, &
&                           PHIAW=INTFLDS%PHIAW)
    CALL SRC_CONTRIBS%INIT(XLLWS=XLLWS, MIJ=MIJ)

!$loki update_device

      CALL WVPRPT_FIELD%UPDATE_DEVICE(WAVNUM=WAVNUM_DPTR, CGROUP=CGROUP_DPTR, CIWA=CIWA_DPTR, CINV=CINV_DPTR, XK2CG=XK2CG_DPTR,  &
      & STOKFAC=STOKFAC_DPTR)
      CALL WVENVI_FIELD%UPDATE_DEVICE(EMAXDPT=EMAXDPT_DPTR, INDEP=INDEP_DPTR, DEPTH=DEPTH_DPTR, IOBND=IOBND_DPTR, IODP=IODP_DPTR)
      CALL FF_NOW_FIELD%UPDATE_DEVICE(AIRD=AIRD_DPTR, WDWAVE=WDWAVE_DPTR, CICOVER=CICOVER_DPTR, WSWAVE=WSWAVE_DPTR,  &
      & WSTAR=WSTAR_DPTR, UFRIC=UFRIC_DPTR, TAUW=TAUW_DPTR, TAUWDIR=TAUWDIR_DPTR, Z0M=Z0M_DPTR, Z0B=Z0B_DPTR,  &
      & CHRNCK=CHRNCK_DPTR, CITHICK=CITHICK_DPTR)
      CALL WAM2NEMO_FIELD%UPDATE_DEVICE(NEMOUSTOKES=NEMOUSTOKES_DPTR, NEMOVSTOKES=NEMOVSTOKES_DPTR, NEMOSTRN=NEMOSTRN_DPTR,  &
      & NPHIEPS=NPHIEPS_DPTR, NTAUOC=NTAUOC_DPTR, NSWH=NSWH_DPTR, NMWP=NMWP_DPTR, NEMOTAUX=NEMOTAUX_DPTR,  &
      & NEMOTAUY=NEMOTAUY_DPTR, NEMOWSWAVE=NEMOWSWAVE_DPTR, NEMOPHIF=NEMOPHIF_DPTR)
      CALL INTFLDS_FIELD%UPDATE_DEVICE(WSEMEAN=WSEMEAN_DPTR, WSFMEAN=WSFMEAN_DPTR, USTOKES=USTOKES_DPTR,  &
      & VSTOKES=VSTOKES_DPTR, STRNMS=STRNMS_DPTR, TAUXD=TAUXD_DPTR, TAUYD=TAUYD_DPTR, TAUOCXD=TAUOCXD_DPTR,  &
      & TAUOCYD=TAUOCYD_DPTR, TAUOC=TAUOC_DPTR, PHIOCD=PHIOCD_DPTR, PHIEPS=PHIEPS_DPTR, PHIAW=PHIAW_DPTR)
      CALL SRC_CONTRIBS%UPDATE_DEVICE(XLLWS=XLLWS_DPTR, MIJ=MIJ_DPTR)

!$acc data present(FL1_DPTR,XLLWS_DPTR,MIJ_DPTR,WAVNUM_DPTR,CGROUP_DPTR,CIWA_DPTR,CINV_DPTR,XK2CG_DPTR,STOKFAC_DPTR,&
!$acc &            EMAXDPT_DPTR,INDEP_DPTR,DEPTH_DPTR,IOBND_DPTR,IODP_DPTR,CICOVER_DPTR,WSWAVE_DPTR,WDWAVE_DPTR,AIRD_DPTR,&
!$acc &            WSTAR_DPTR,UFRIC_DPTR,TAUW_DPTR,TAUWDIR_DPTR,Z0M_DPTR,Z0B_DPTR,CHRNCK_DPTR,CITHICK_DPTR,NEMOUSTOKES_DPTR,&
!$acc &            NEMOVSTOKES_DPTR,NEMOSTRN_DPTR,NPHIEPS_DPTR,NTAUOC_DPTR,NSWH_DPTR,NMWP_DPTR,NEMOTAUX_DPTR,NEMOTAUY_DPTR,&
!$acc &            NEMOWSWAVE_DPTR,NEMOPHIF_DPTR,WSEMEAN_DPTR,WSFMEAN_DPTR,USTOKES_DPTR,VSTOKES_DPTR,STRNMS_DPTR,TAUXD_DPTR,&
!$acc &            TAUYD_DPTR,TAUOCXD_DPTR,TAUOCYD_DPTR,TAUOC_DPTR,PHIOCD_DPTR,PHIEPS_DPTR,PHIAW_DPTR)
      TIME0=-WAM_USER_CLOCK()
!$acc parallel loop gang vector_length(NPROMA_WAM)
      DO ICHNK=1,NCHNK
        CALL IMPLSCH(1, NPROMA_WAM, FL1_DPTR(:,:,:,ICHNK), WAVNUM_DPTR(:,:,ICHNK), &
        & CGROUP_DPTR(:,:,ICHNK), CIWA_DPTR(:,:,ICHNK), CINV_DPTR(:,:,ICHNK), &
        & XK2CG_DPTR(:,:,ICHNK), STOKFAC_DPTR(:,:,ICHNK), EMAXDPT_DPTR(:,ICHNK), &
        & INDEP_DPTR(:,ICHNK), DEPTH_DPTR(:,ICHNK), IOBND_DPTR(:,ICHNK), &
        & IODP_DPTR(:,ICHNK), AIRD_DPTR(:,ICHNK), WDWAVE_DPTR(:,ICHNK),  &
        & CICOVER_DPTR(:,ICHNK), WSWAVE_DPTR(:,ICHNK), WSTAR_DPTR(:,ICHNK), &
        & UFRIC_DPTR(:,ICHNK), TAUW_DPTR(:,ICHNK), TAUWDIR_DPTR(:,ICHNK), &
        & Z0M_DPTR(:,ICHNK), Z0B_DPTR(:,ICHNK), CHRNCK_DPTR(:,ICHNK), &
        & CITHICK_DPTR(:,ICHNK), NEMOUSTOKES_DPTR(:,ICHNK), NEMOVSTOKES_DPTR(:,ICHNK), &
        & NEMOSTRN_DPTR(:,ICHNK), NPHIEPS_DPTR(:,ICHNK), NTAUOC_DPTR(:,ICHNK), &
        & NSWH_DPTR(:,ICHNK), NMWP_DPTR(:,ICHNK), NEMOTAUX_DPTR(:,ICHNK), &
        & NEMOTAUY_DPTR(:,ICHNK), NEMOWSWAVE_DPTR(:,ICHNK), NEMOPHIF_DPTR(:,ICHNK), &
        & WSEMEAN_DPTR(:,ICHNK), WSFMEAN_DPTR(:,ICHNK), USTOKES_DPTR(:,ICHNK), &
        & VSTOKES_DPTR(:,ICHNK), STRNMS_DPTR(:,ICHNK), TAUXD_DPTR(:,ICHNK), &
        & TAUYD_DPTR(:,ICHNK), TAUOCXD_DPTR(:,ICHNK), TAUOCYD_DPTR(:,ICHNK), &
        & TAUOC_DPTR(:,ICHNK), PHIOCD_DPTR(:,ICHNK), PHIEPS_DPTR(:,ICHNK),  &
        & PHIAW_DPTR(:,ICHNK), MIJ_DPTR(:,ICHNK), XLLWS_DPTR(:,:,:,ICHNK))
      END DO
!$acc end parallel loop
      TIME1(2) = TIME1(2) + (TIME0+WAM_USER_CLOCK())*1.E-06
!$acc end data
      CALL WVPRPT_FIELD%ENSURE_HOST()
      CALL WVENVI_FIELD%ENSURE_HOST()
      CALL FF_NOW_FIELD%ENSURE_HOST()
      CALL WAM2NEMO_FIELD%ENSURE_HOST()
      CALL INTFLDS_FIELD%ENSURE_HOST()
      CALL SRC_CONTRIBS%ENSURE_HOST()

!$loki update_host
    TIME1(3) = TIME1(3) + (TIME2+WAM_USER_CLOCK())*1.E-06

    IF (LWNEMOCOU) NEMONTAU = NEMONTAU + 1

  ELSE
!   NO SOURCE TERM CONTRIBUTION
!$OMP      PARALLEL DO SCHEDULE(STATIC) PRIVATE(ICHNK)  
    DO ICHNK = 1, NCHNK
      MIJ(:,ICHNK) = NFRE
      FL1(:,:,:,ICHNK) = MAX(FL1(:,:,:,ICHNK), EPSMIN)
      XLLWS(:,:,:,ICHNK) = 0.0_JWRB
    ENDDO
!$OMP      END PARALLEL DO
  ENDIF
  CALL GSTATS(1431,1)


!*       UPDATE FORCING FIELDS TIME COUNTER 
!        ----------------------------------
  IF (LLNEWFILE) THEN
    LLNEWFILE = .FALSE.
    IDELWH = MAX(IDELWI, IDELPRO)
    CALL INCDATE(CDAWIFL, IDELWH)
    CALL INCDATE(CDATEFL, IDELWH)
  ENDIF

  CDATEWO=CDATEWH
  CDTIMP=CDTIMPNEXT
  CALL INCDATE(CDTIMPNEXT, IDELT)   

ENDIF

IF (LHOOK) CALL DR_HOOK('WAMINTGR',1,ZHOOK_HANDLE)

END SUBROUTINE WAMINTGR_LOKI_GPU
