! (C) Copyright 1989- ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!

      MODULE YOWALTAS

      USE PARKIND_WAVE, ONLY : JWIM, JWRB, JWRU

      IMPLICIT NONE

      INTEGER(KIND=JWIM), PARAMETER :: NUMALT=10
      INTEGER(KIND=JWIM), PARAMETER :: NIJALT=4
      INTEGER(KIND=JWIM), PARAMETER :: NALTDT=3
      INTEGER(KIND=JWIM), PARAMETER :: NALTEDT=3
      INTEGER(KIND=JWIM), PARAMETER :: NALTUDT=2

      INTEGER(KIND=JWIM) :: NALTAVLB
      INTEGER(KIND=JWIM) :: IBUFRSAT(NUMALT)
      INTEGER(KIND=JWIM), ALLOCATABLE :: IJALT(:,:)
      INTEGER(KIND=JWIM), ALLOCATABLE :: INTLMAX(:)
      INTEGER(KIND=JWIM), ALLOCATABLE :: KMINLMAX(:)
      INTEGER(KIND=JWIM), ALLOCATABLE :: KMAXLMAX(:)
      INTEGER(KIND=JWIM), ALLOCATABLE :: NOBSPE(:)

      REAL(KIND=JWRB) :: EGRCRV
      REAL(KIND=JWRB) :: AGRCRV
      REAL(KIND=JWRB) :: BGRCRV
      REAL(KIND=JWRB) :: AFCRV
      REAL(KIND=JWRB) :: BFCRV
      REAL(KIND=JWRB) :: ESH
      REAL(KIND=JWRB) :: ASH
      REAL(KIND=JWRB) :: BSH
      REAL(KIND=JWRB) :: ASWKM
      REAL(KIND=JWRB) :: BSWKM

      REAL(KIND=JWRB) :: XKAPPA2(NUMALT)
      REAL(KIND=JWRB) :: HSCOEFCOR(NUMALT)
      REAL(KIND=JWRB) :: HSCONSCOR(NUMALT)
      REAL(KIND=JWRB) :: ALTSDTHRSH(NUMALT)
      REAL(KIND=JWRB) :: ALTBGTHRSH(NUMALT)
      REAL(KIND=JWRB) :: HSALTCUT(NUMALT)

      REAL(KIND=JWRB), ALLOCATABLE :: ALTDATA(:,:)
      REAL(KIND=JWRB), ALLOCATABLE :: ALTEXDATA(:,:)
      REAL(KIND=JWRB), ALLOCATABLE, DIMENSION(:) :: XLONOBS, SIGRATIO2, DIFFALTFG

      REAL(KIND=JWRU), ALLOCATABLE :: ALTUNDATA(:,:)

      CHARACTER(LEN=25) :: CSATNAME(NUMALT) 
      CHARACTER(LEN=14), ALLOCATABLE, DIMENSION(:) :: CDATEOBS

      LOGICAL :: LODBRALT
      LOGICAL :: LALTCOR(NUMALT)
      LOGICAL :: LALTLRGR(NUMALT)
      LOGICAL :: LALTGRDOUT(NUMALT)
      LOGICAL :: LALTPAS(NUMALT)
      LOGICAL, ALLOCATABLE :: LALTPASSIV(:)

!     VARIABLE   TYPE     PURPOSE                                  
!     --------   ----     -------                                  
!     *NUMALT*   INTEGER  MAXIMUM NUMBER OF ALTIMETERS ALLOWED IN ALTAS.
!     *NIJALT*   INTEGER  SECOND DIMENSION OF IJALT
!     *NALTDT*   INTEGER  SECOND DIMENSION OF ALTDATA
!                         IF 1 : CORRECTED WAVE HEIGHTS
!                            2 : ERROR ASSOCIATED WITH THE DATUM
!                            3 : ORIGINAL WAVE HEIGHT
!     *NALTEDT*  INTEGER  SECOND DIMENSION OF ALTEXDATA
!                            1 : LATITUDE OF OBSERVATION
!                            2 : LONGITUDE OF OBSERVATION
!                            3 : ORIGINAL WIND SPEED
!     *NALTUDT*  INTEGER  SECOND DIMENSION OF ALTUNDATA
!                            1 : LATITUDE OF CLOSEST NODE TO OBSERVATION
!                            2 : LONGITUDE OF CLOSEST NODE OBSERVATION
!     *NALTAVLB* INTEGER  NUMBER OF ALTIMETERS FROM WHICH THERE ARE 
!                         AVAILABLE DATA.
!     *IBUFRSAT* INTEGER  BUFR IDENTIFIER FOR THE SATELLITES
!     *IJALT*    INTEGER  ARRAY FOR: 1: ALTIMETER DATA BLOCK INDEX,
!                                    2: SATELLITE IDENTIFIER,  AND
!                                    3: WAVE HEIGHT QUALITY STATUS
!                                    4: WIND SPEED QUALITY STATUS
!                                      STATUS:
!                                      1: ACTIVE
!                                      0: PASSIVE OR BLACKLISTED (passing all QC's)
!                                     -1: MISSING DATA
!                                     -2: TOO SMALL (BELOW A CERTAIN THRESHOLD) 
!                                     -3: TOO FEW MEASUREMENTS TO CREATE SUPEROBS
!                                     -4: STANDARD DEVIATION TOO LARGE TO CREATE SUPEROBS
!                                     -5: OVER MODEL SEA ICE THRESHOLD (SEE CITHRSH_SAT)
!                                     -6: FAILED BACKGROUNG CHECK

!     *INTLMAX*  INTEGER  TABLE INDICATING WHETHER A PE COULD SHARE
!                         ALTIMETER DATA WITH LOCAL PE (IRANK) 
!     *KMINLMAX* INTEGER  SOUTHERN LATITUDE INDEX OF THE SOUTHERN
!                         LATITUDONAL BAND OF WIDTH LMAX FOR EACH PE
!     *KMAXLMAX* INTEGER  NORTHERN LATITUDE INDEX OF THE NORTHERN
!                         LATITUDONAL BAND OF WIDTH LMAX FOR EACH PE
!     *NOBSPE*   INTEGER  NUMBER OF DATA NEEDED PER PE (it includes data in the communication hallo).

!!!   EMPIRICAL CONSTANCE FOR  SPECTRAL UPDATE FOLLOWING DATA ASSIMILATION
!     *EGRCRV*    REAL      PARAMETER OF THE NON DIMENSIONAL ENERGY 
!                           GROWTH CURVE.
!                           ESTAR=EGRCRV*(TSTAR/(AGRCRV+TSTAR))**BGRCRV
!     *AGRCRV*    REAL      PARAMETER OF THE NON DIMENSIONAL ENERGY 
!                           GROWTH CURVE.
!     *BGRCRV*    REAL      PARAMETER OF THE NON DIMENSIONAL ENERGY 
!                           GROWTH CURVE.
!     *AFCRV*     REAL      PARAMETER OF THE NON DIMENSIONAL FREQUENCY 
!                           GROWTH CURVE. Based on f * F(f)
!                           FSTAR=(EGRCRV/AFCRV)**(1/BFCRV)
!     *BFCRV*     REAL      PARAMETER OF THE NON DIMENSIONAL FREQUENCY
!                           GROWTH CURVES.
!     *AFMRV*     REAL      PARAMETER OF THE NON DIMENSIONAL FREQUENCY 
!                           GROWTH CURVE. Based on 1/f F(f)
!                           FSTAR=(EGRCRV/AFCRV)**(1/BFCRV)
!     *BFMRV*     REAL      PARAMETER OF THE NON DIMENSIONAL FREQUENCY
!                           GROWTH CURVE.
!     *ESH*       REAL      PARAMETER OF THE NON DIMENSIONAL ENERGY LIMIT
!                           AS A FUNCTION OF NON-DIMENSIONAL DEPTH DSTAR
!                           ESTAR_LIMIT=ESH*TANH(ASH*DSTAR**BSH), where
!                           DSTAR=DEPTH*G/USTAR**2
!     *ASH*       REAL      PARAMETER OF THE NON DIMENSIONAL ENERGY LIMIT
!                           SEE ESH.
!     *BSH*       REAL      PARAMETER OF THE NON DIMENSIONAL ENERGY LIMIT
!                           SEE ESH.
!     *ASWKM*     REAL      PARAMETER TO EXPRESS MINIMUM KMEAN AS A FUNCTION
!                           OF DEPTH: KMEAN >= ASWKM/DEPTH**BSWKM 
!     *BSWKM*     REAL      PARAMETER TO EXPRESS MINIMUM KMEAN AS A FUNCTION
!                           OF DEPTH: KMEAN >= ASWKM/DEPTH**BSWKM 

!     *XKAPPA2*  REAL     KAPPA2 PARAMETER USED IN THE DETERMINATION
!                         OF ALTIMETER WAVE HEIGHTS (SEE GRFIELD)
!     *HSCOEFCOR*REAL     COEFFICIENT OF THE CORRECTIVE LINEAR
!                         REGRESSION FOR ALTIMETER WAVE HEIGHTS.
!     *HSCONSCOR*REAL     CONSTANT COEFFICIE$NT  OF THE CORRECTIVE
!                         LINEAR REGRESSION FOR ALTIMETER WAVE
!                         HEIGHTS.
!     *ALTSDTHRSH* REAL   THRESHOLD FOR SUSPICIOUS DATA (SEE GRFIELD).
!     *ALTBGTHRSH* REAL   THRESHOLD FOR BACKGROUND CHECK (SEE GRFIELD).
!     *HSALTCUT* REAL     USER INPUT OF THE MINIMUM WAVE HEIGHT ALLOWED
!                         FOR THE ALTIMETER WAVE HEIGHT. THE ACTUAL
!                         MINIMUM HSCUT=MIN(HSALTCUT,SIGMA_ALT)
!                         SEE GRFIELD. 
!     *ALTDATA*  REAL     ALTIMETER DATA AND ERROR ESTIMATE AFTER
!                         PREPROCESSING
!     *ALTEXDATA* REAL    EXTRA ALTIMETER DATA (THAT WILL NOT BE PASSED 
!                         TO THE MODEL .. NEEDED ONLY FOR ODB CREATION)
!     *XLONOBS*  REAL     LONGITUDE OF ALTIMETER DATA AS USED BY OI.
!     *SIGRATIO2*REAL     RATIO OF ALTIMETER TO MODEL ERROR IN OI.
!     *DIFFALTFG*REAL     OBS MINUS FIRST GUESS AT OBS LOCATIONS.

!     *ALTUNDATA* REAL8   FOR UNSTRUCTURED GRID: THE COORDINATES OF THE CLOSEST NODE

!     *CSATNAME*  CHARACTER NAME OF THE DIFFERENT ALTIMETERS
!     *LODBRALT* LOGICAL  CONTROLS THE USE OF ODB FOR RADAR ALTIMETER OBS.
!     *LALTCOR*  LOGICAL  CONTROLS WHETHER THE ALTIMETER WAVE HEIGHT
!                         OBSERVATION IS CORRECTED USING THE WAVE
!                         SEA STATE PRIOR TO ITS ASSIMILATION.
!     *LALTLRGR* LOGICAL  CONTROLS WHETHER THE ALTIMETER WAVE HEIGHT
!                         OBSERVATION IS CORRECTED USING A PRESCRIBED
!                         LINEAR REGRESSION.
!     *LALTGRDOUT* LOGICAL  CONTROLS WHETHER TO OUTPUT THE GRIDDED
!                           ALTIMETER PRODUCTS OF THE SPECIFIC
!                           INSTRUMENT (AS PROVIDED IN THE NAME LIST)
!     *LALTPAS* LOGICAL   CONTROLS WHETHER OR NOT THE DATA ARE FED 
!                         PASSIVELY OR NOT (DEFAULT).
!     *LALTPASSIV*LOGICAL USE BY OIFIELD TO EXCLUDE PASSIVE DATA. 

! ----------------------------------------------------------------------
      END MODULE YOWALTAS
