! (C) Copyright 1989- ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!

!
! MODULE YOWGSTATS is used to set up gstats in the context outside of the IFS
!

MODULE YOWGSTATS
IMPLICIT NONE
PRIVATE
PUBLIC :: WAM_GSTATS_SETUP
PUBLIC :: WAM_GSTATS_PRINT
PUBLIC :: WAM_GSTATS_FILE_OPEN
PUBLIC :: WAM_GSTATS_FILE_CLOSE

CONTAINS

! ------------------------------------------------------------------------------------------

SUBROUTINE WAM_GSTATS_SETUP(KUOUT)
! ------------------------------------------------------------------------------------------

!**** *WAM_GSTATS_SETUP* - 

!*    PURPOSE.
!     --------

!     SET UP GSTATS IN STANDALONE CONTEXT FROM RUNWAM

!     ENVIRONMENT VARIABLES
!     ---------------------
!     GSTATS         : TRUE/FALSE/ON/OFF/0/1 to enable GSTATS
!     GSTATS_DISABLE : To disable GSTATS components
!     GSTATS_ENABLE  : To enable GSTATS components

!     OPTIONAL INPUT ARGUMENTS
!     ------------------------
!     KUOUT : Output unit for logging setup.
!             If not specified, unit 6 is used on RANK 1

! ------------------------------------------------------------------------------------------
USE MPL_MODULE, ONLY : MPL_MYRANK, MPL_NPROC
USE EC_PARKIND, ONLY : JPIM, JPRD
IMPLICIT NONE
! ----------------------------------------------------------------------
INTEGER(KIND=JPIM), INTENT(IN), OPTIONAL :: KUOUT
! ----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: MYPROC, NPROC, JPROC, IUOUT
INTEGER(KIND=JPIM), ALLOCATABLE :: NPRCIDS(:)

LOGICAL :: LGSTATS     = .FALSE.
LOGICAL :: LGSTATS_ALL = .FALSE.

LOGICAL            :: LDETAILED_STATS = .TRUE.
LOGICAL            :: LTRACE_STATS    = .FALSE.
LOGICAL            :: LSTATS_OMP      = .TRUE.
LOGICAL            :: LSTATS_COMMS    = .TRUE.
LOGICAL            :: LBARRIER_STATS  = .FALSE.
LOGICAL            :: LBARRIER_STATS2 = .FALSE.
LOGICAL            :: LSTATS_ALLOC    = .FALSE.
LOGICAL            :: LSYNCSTATS      = .FALSE.
LOGICAL            :: LSTATSCPU       = .TRUE.
LOGICAL            :: LSTATS_MEM      = .FALSE.
LOGICAL            :: LXML_STATS      = .FALSE.
INTEGER(KIND=JPIM) :: NSTATS_MEM      = 0
INTEGER(KIND=JPIM) :: NTRACE_STATS    = 0
INTEGER(KIND=JPIM) :: NPRNT_STATS     = 1

CHARACTER(LEN=512) :: CLENV

#include "gstats_setup.intfb.h"
! ----------------------------------------------------------------------

NPROC  = MPL_NPROC()
MYPROC = MPL_MYRANK()
ALLOCATE(NPRCIDS(NPROC))
DO JPROC = 1, NPROC
  NPRCIDS(JPROC) = JPROC
ENDDO

IF (PRESENT(KUOUT)) THEN
  IUOUT = KUOUT
ELSE
  IUOUT = -1
  IF( MPL_MYRANK() == 1 ) IUOUT = 6
ENDIF

LGSTATS = GET_LOGICAL_ENVIRONMENT_VARIABLE("GSTATS",.FALSE.)

IF (IUOUT /= -1) THEN
  WRITE(IUOUT,*) "WAM_GSTATS_SETUP:"
  WRITE(IUOUT,*) "   ${GSTATS}         : ", LGSTATS
ENDIF

CALL GET_ENVIRONMENT_VARIABLE("GSTATS_NPROC",CLENV)
IF (IUOUT /= -1) THEN
  WRITE(IUOUT,*) "   ${GSTATS_NPROC}   : ", TRIM(CLENV)
ENDIF

IF ( CLENV == 'ALL' ) THEN
  NPRNT_STATS = NPROC
ELSEIF ( CLENV /= '' ) THEN
  READ(CLENV,*) NPRNT_STATS
ENDIF

CALL GET_ENVIRONMENT_VARIABLE("GSTATS_ENABLE",CLENV)
IF (IUOUT /= -1) THEN
  WRITE(IUOUT,*) "   ${GSTATS_ENABLE}  : ", TRIM(CLENV)
ENDIF

CALL GET_ENVIRONMENT_VARIABLE("GSTATS_DISABLE",CLENV)
IF (IUOUT /= -1) THEN
  WRITE(IUOUT,*) "   ${GSTATS_DISABLE} : ", TRIM(CLENV)
ENDIF

IF (LGSTATS) THEN
  IF (STRCONTAINS(CLENV,'DETAIL')) LDETAILED_STATS = .FALSE.
  IF (STRCONTAINS(CLENV,'OMP'))    LSTATS_OMP      = .FALSE.
  IF (STRCONTAINS(CLENV,'COMM'))   LSTATS_COMMS    = .FALSE.
  IF (STRCONTAINS(CLENV,'CPU'))    LSTATSCPU       = .FALSE.
  IF (STRCONTAINS(CLENV,'BAR'))    LBARRIER_STATS  = .FALSE.
  IF (STRCONTAINS(CLENV,'BAR'))    LBARRIER_STATS2 = .FALSE.
  IF (STRCONTAINS(CLENV,'MEM'))    LSTATS_MEM      = .FALSE.
  IF (STRCONTAINS(CLENV,'ALLOC'))  LSTATS_ALLOC    = .FALSE.
  IF (STRCONTAINS(CLENV,'SYNC'))   LSYNCSTATS      = .FALSE.
  IF (STRCONTAINS(CLENV,'TRACE'))  LTRACE_STATS    = .FALSE.

  CALL GET_ENVIRONMENT_VARIABLE("GSTATS_ENABLE",CLENV)

  IF (STRCONTAINS(CLENV,'DETAIL')) LDETAILED_STATS = .TRUE.
  IF (STRCONTAINS(CLENV,'DETAIL')) NPRNT_STATS     =  NPROC
  IF (STRCONTAINS(CLENV,'OMP'))    LSTATS_OMP      = .TRUE.
  IF (STRCONTAINS(CLENV,'COMM'))   LSTATS_COMMS    = .TRUE.
  IF (STRCONTAINS(CLENV,'CPU'))    LSTATSCPU       = .TRUE.
  IF (STRCONTAINS(CLENV,'BAR'))    LBARRIER_STATS  = .TRUE.
  IF (STRCONTAINS(CLENV,'BAR'))    LBARRIER_STATS2 = .TRUE.
  IF (STRCONTAINS(CLENV,'MEM'))    LSTATS_MEM      = .TRUE.
  IF (STRCONTAINS(CLENV,'ALLOC'))  LSTATS_ALLOC    = .TRUE.
  IF (STRCONTAINS(CLENV,'SYNC'))   LSYNCSTATS      = .TRUE.
  IF (STRCONTAINS(CLENV,'TRACE'))  LTRACE_STATS    = .TRUE.
  IF (STRCONTAINS(CLENV,'XML'))    LXML_STATS      = .TRUE.

  IF (IUOUT /= -1) THEN
    WRITE(IUOUT,*) "GSTATS CONFIGURED OPTIONS:"
    WRITE(IUOUT,*) "   DETAILED : ", LDETAILED_STATS
    WRITE(IUOUT,*) "   OMP      : ", LSTATS_OMP
    WRITE(IUOUT,*) "   COMMS    : ", LSTATS_OMP
    WRITE(IUOUT,*) "   CPU      : ", LSTATS_OMP
    WRITE(IUOUT,*) "   BARRIER  : ", LBARRIER_STATS
    WRITE(IUOUT,*) "   MEM      : ", LSTATS_MEM
    WRITE(IUOUT,*) "   ALLOC    : ", LSTATS_ALLOC
    WRITE(IUOUT,*) "   SYNC     : ", LSYNCSTATS
    WRITE(IUOUT,*) "   XML      : ", LXML_STATS
    WRITE(IUOUT,*) "   TRACE    : ", LTRACE_STATS
    WRITE(IUOUT,*) "   NPRNT    : ", NPRNT_STATS
  ENDIF
ENDIF

CALL GSTATS_SETUP(NPROC, MYPROC, NPRCIDS,                                                &
    & LGSTATS, LSTATSCPU, LSYNCSTATS, LDETAILED_STATS, LBARRIER_STATS, LBARRIER_STATS2, &
    & LSTATS_OMP, LSTATS_COMMS, LSTATS_MEM, NSTATS_MEM, LSTATS_ALLOC,                   &
    & LTRACE_STATS, NTRACE_STATS, NPRNT_STATS, LXML_STATS)

CALL WAM_GSTATS_SETUP_LABELS() ! See below

CALL GSTATS_PSUT() ! MEASURES PARALLEL START UP TIME

END SUBROUTINE WAM_GSTATS_SETUP

! ------------------------------------------------------------------------------------------

SUBROUTINE WAM_GSTATS_PRINT(KULOUT)
USE EC_PARKIND, ONLY : JPIM, JPRD
USE YOMGSTATS, ONLY : JPMAXSTAT, LSTATS
INTEGER(KIND=JPIM), INTENT(IN) :: KULOUT
REAL(KIND=JPRD) :: ZAVEAVE(0:JPMAXSTAT)
IF (LSTATS) THEN
  CALL GSTATS_PRINT(KULOUT,ZAVEAVE,JPMAXSTAT)
ENDIF
END SUBROUTINE WAM_GSTATS_PRINT

! ------------------------------------------------------------------------------------------

FUNCTION GET_LOGICAL_ENVIRONMENT_VARIABLE(CKEY,LDEFAULT) RESULT(LVALUE)
    LOGICAL :: LVALUE
    CHARACTER(LEN=*), INTENT(IN) :: CKEY
    LOGICAL, INTENT(IN) :: LDEFAULT
    CHARACTER(LEN=512) :: CLENV
    CALL GET_ENVIRONMENT_VARIABLE(CKEY,CLENV)
    LVALUE = LDEFAULT
    IF ( CLENV == '1'      .OR. &
       & CLENV(1:1) == 't' .OR. CLENV(1:1) == 'T' .OR. &
       & CLENV(1:1) == 'y' .OR. CLENV(1:1) == 'Y' .OR. &
       & CLENV == 'on'     .OR. CLENV == 'ON' ) THEN
       LVALUE = .TRUE.
    ENDIF
    IF ( CLENV == '0'      .OR. &
       & CLENV(1:1) == 'f' .OR. CLENV(1:1) == 'F' .OR. &
       & CLENV(1:1) == 'n' .OR. CLENV(1:1) == 'N' .OR. &
       & CLENV == 'off'    .OR. CLENV == 'OFF' ) THEN
       LVALUE = .FALSE.
    ENDIF
END FUNCTION GET_LOGICAL_ENVIRONMENT_VARIABLE

! ------------------------------------------------------------------------------------------

PURE LOGICAL FUNCTION STRCONTAINS(CSTR,CSUBSTR)
     CHARACTER(LEN=*), INTENT(IN) :: CSTR, CSUBSTR
     STRCONTAINS = ( INDEX(CSTR,CSUBSTR) /= 0 )
END FUNCTION

! ------------------------------------------------------------------------------------------

SUBROUTINE WAM_GSTATS_SETUP_LABELS
USE YOMGSTATS, ONLY : LGSTATS_LABEL
IMPLICIT NONE
! For now copy of IFS, serves also to get idea of available labels
! Note that this routine is not called in context of IFS
! IFS's gstats_labels must be updated accordingly

CALL GSTATS_LABEL(0 ,'   ','TOTAL') ! THIS IS CNT0 in IFS
CALL GSTATS_LABEL(1 ,'   ','CNT4     - FORWARD INTEGRATION')
CALL GSTATS_LABEL(7 ,'   ','SCAN2M - MASS FIXERS TOTAL')
CALL GSTATS_LABEL(8 ,'   ','SCAN2M - GRID-POINT DYNAMICS')
CALL GSTATS_LABEL(9 ,'   ','SPCM     - SPECTRAL COMP.')
CALL GSTATS_LABEL(10,'   ','SCAN2M - PHYSICS')
CALL GSTATS_LABEL(11,'   ','IOPACK   - OUTPUT P.P. RESULTS')
CALL GSTATS_LABEL(12,'   ','SPNORM   - SPECTRAL NORM COMP.')
CALL GSTATS_LABEL(13,'   ','SCAN2M - RADIATION CALC.')
CALL GSTATS_LABEL(14,'   ','SUINIF ')
CALL GSTATS_LABEL(15,'   ','GET_TRAJ_SPEC ')
CALL GSTATS_LABEL(16,'   ','GET_TRAJ_GRID ')
CALL GSTATS_LABEL(17,'   ','GRIDFPOS IN CNT4 ')
CALL GSTATS_LABEL(18,'   ','SUSPECG ')
CALL GSTATS_LABEL(19,'   ','SUSPEC ')
CALL GSTATS_LABEL(20,'   ','CVA1     - CONTROL OF MINIM.')
CALL GSTATS_LABEL(21,'   ','SIM4D    - FULL COST FUNCTION')
CALL GSTATS_LABEL(22,'   ','CNT4TL   - TANGENT LINEAR INT.')
CALL GSTATS_LABEL(23,'   ','CNT4AD   - ADJOINT INT.')
CALL GSTATS_LABEL(24,'   ','SUGRIDU ')
CALL GSTATS_LABEL(25,'   ','SPECRT ')
CALL GSTATS_LABEL(26,'   ','SUGRIDF ')
CALL GSTATS_LABEL(27,'   ','RESTART FILES - WRITING')
CALL GSTATS_LABEL(28,'   ','RESTART FILES - READING')
CALL GSTATS_LABEL(29,'   ','SU4FPOS IN CNT4 ')
CALL GSTATS_LABEL(30,'   ','DYNFPOS IN CNT4 ')
CALL GSTATS_LABEL(31,'   ','POSDDH IN STEPO ')
CALL GSTATS_LABEL(32,'   ','NEMO     - COST OF OCEAN MODEL')
CALL GSTATS_LABEL(33,'   ','NEMO     - IFS ACCUM. OF DATA')
CALL GSTATS_LABEL(34,'   ','SCAN2MTL - T.L. GRID-POINT COMP.')
CALL GSTATS_LABEL(35,'   ','SCAN2MAD - ADJ. GRID-POINT COMP.')
CALL GSTATS_LABEL(36,'   ','SPCMAD   - ADJOINT SPECTRAL COMP.')
CALL GSTATS_LABEL(37,'   ','CPGLAG   - SL COMPUTATIONS')
CALL GSTATS_LABEL(38,'   ','WAM      - TOTAL COST OF WAVE MODEL')
CALL GSTATS_LABEL(39,'   ','SU0YOMB ')
CALL GSTATS_LABEL(40,'   ','OBSV     - OBSERVATION CALC.')
CALL GSTATS_LABEL(41,'   ','OBSVTL   - T.L. OBSERVATION CALC.')
CALL GSTATS_LABEL(42,'   ','OBSVAD   - ADJ. OBSERVATION CALC.')
CALL GSTATS_LABEL(43,'   ','OBSHOR   - OBS. EQUIV. COMP.')
CALL GSTATS_LABEL(44,'   ','OBSHORAD - ADJ. OBS. EQUIV. COMP.')
CALL GSTATS_LABEL(45,'   ','CNT1     - SCREENING')
CALL GSTATS_LABEL(46,'   ','CNT1     - OBSERVATION WRITING')
CALL GSTATS_LABEL(47,'   ','CVA1     - OBSERVATION WRITING')
CALL GSTATS_LABEL(48,'   ','SIM4D    - COST FUNCTION EVAL.')
CALL GSTATS_LABEL(49,'   ','SIM4D    - JB CALCULATION')
CALL GSTATS_LABEL(50,'   ','SIM4D    - CHANGE OF VAR.')

!   counters 51 to 100 :  message passing 

CALL GSTATS_LABEL(51,'   ','SCAN2M   - SL COMM. PART 1')
CALL GSTATS_LABEL(54,'   ','SPCM     - M TO S/S TO M TRANSP.')
CALL GSTATS_LABEL(55,'   ','SPCIMPF  - S TO M/M TO S TRANSP.')
CALL GSTATS_LABEL(56,'   ','SPNORM   - SPECTRAL NORM COMM.')
CALL GSTATS_LABEL(59,'   ','SLINT    - SL COMM.')
CALL GSTATS_LABEL(63,'   ','SCAN2MAD - SL COMM. PART 1')
CALL GSTATS_LABEL(64,'   ','SCAN2MAD - SL COMM. PART 2')
CALL GSTATS_LABEL(65,'   ','RADINTG  - SL COMM. OUTPUT')
CALL GSTATS_LABEL(66,'   ','RADINTG  - SL COMM. INPUT')
CALL GSTATS_LABEL(67,'   ','SLCOMM COAPHY-1 ')
CALL GSTATS_LABEL(68,'   ','SLCOMM COAPHY-2 ')

CALL GSTATS_LABEL(71,'   ','IOSERV   - RECLAIM BUF SPACE')
CALL GSTATS_LABEL(72,'   ','IOSERV   - SEND MDL')

CALL GSTATS_LABEL(79,'   ','THINN    - DECIS/OBS.THINNING (AIRS ONLY)')

CALL GSTATS_LABEL(84,'   ','SCAN2MTL - SL COMM.')
CALL GSTATS_LABEL(85,'   ','SCAN2MAD - INC SL COMM.')
CALL GSTATS_LABEL(86,'   ','SPCMAD   - M TO S/S TO M TRANSP.')
CALL GSTATS_LABEL(87,'   ','SPCIMPFAD- S TO M/M TO S TRANSP.')
CALL GSTATS_LABEL(88,'   ','SCAN2MTL - SL COMM. PART 2')
CALL GSTATS_LABEL(89,'   ','SCAN2M - SL COMM. PART 2')
CALL GSTATS_LABEL(90,'   ','OBSV     - COMM.')
CALL GSTATS_LABEL(91,'   ','OBSVTL   - COMM.')
CALL GSTATS_LABEL(93,'   ','OBSHOR   - COMM. OF OBS == ')
CALL GSTATS_LABEL(94,'   ','OBSHORAD - COMM. OF OBS == ')

!   counters 101 to 200 :  trans package    
CALL GSTATS_LABEL(102,'TRS','LTINV_CTL   - INVERSE LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(103,'TRS','LTDIR_CTL   - DIRECT LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(104,'TRS','LTINV_CTLAD - ADJ. INVERSE LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(105,'TRS','LTDIR_CTLAD - ADJ. DIRECT LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(106,'TRS','FTDIR_CTL   - DIRECT FOURIER TRANSFORM')
CALL GSTATS_LABEL(107,'TRS','FTINV_CTL   - INVERSE FOURIER TRANSFORM')
CALL GSTATS_LABEL(108,'TRS','OMP in GPC  - TEST_ADJOINT ')
CALL GSTATS_LABEL(110,'TRS','OMP in SCALPRODSP  - TEST_ADJOINT ')
CALL GSTATS_LABEL(132,'TRS','FTINV_CTLAD - ADJ. INVERSE FOURIER TRANSFORM')
CALL GSTATS_LABEL(133,'TRS','FTDIR_CTLAD - ADJ. DIRECT FOURIER TRANSFORM')
CALL GSTATS_LABEL(135,'TRS','OMP in GPCAD - TEST_ADJOINT ')
CALL GSTATS_LABEL(140,'TRS','SULEG       - COMP. OF LEGENDRE POL.')
CALL GSTATS_LABEL(152,'TRS','LTINV_CTL   - M TO L TRANSPOSITION')
CALL GSTATS_LABEL(153,'TRS','LTDIR_CTL   - L TO M TRANSPOSITION')
CALL GSTATS_LABEL(157,'TRS','FTINV_CTL   - L TO G TRANSPOSITION')
CALL GSTATS_LABEL(158,'TRS','FTDIR_CTL   - G TO L TRANSPOSITION')
CALL GSTATS_LABEL(180,'TRS','LTINV_CTLAD - L TO M TRANSPOSITION')
CALL GSTATS_LABEL(181,'TRS','LTDIR_CTLAD - M TO L TRANSPOSITION')
CALL GSTATS_LABEL(182,'TRS','FTINV_CTLAD - G TO L TRANSPOSITION')
CALL GSTATS_LABEL(183,'TRS','FTDIR_CTLAD - L TO G TRANSPOSITION')
CALL GSTATS_LABEL(190,'TRS','SUTRLE      - COMMUNICATE LEG.POL.')

!     to be added

!   counters 201 to 250 :  additional computations
CALL GSTATS_LABEL(201,'JBW','JBWAVGEN - INPUT + Control Vector Computations')
CALL GSTATS_LABEL(202,'JBW','JBWAVGEN - GRIDPOINT STDEV Computation')
CALL GSTATS_LABEL(203,'JBW','JBWAVGEN - Hor-Ver. CORRELATION Computation')
CALL GSTATS_LABEL(204,'JBW','JBWAVGEN - NORMALIZATION of CORRELATIONs')
CALL GSTATS_LABEL(205,'JBW','JBWAVGEN - MERGE and EIGENDECOMP.')
CALL GSTATS_LABEL(206,'JBW','JBWAVGEN - WRITE OUT SUJBWAVWRI')
CALL GSTATS_LABEL(207,'JBW','JBWAVVC - COST OF SUJBGPTOMAT')
CALL GSTATS_LABEL(208,'JBW','JBWAVVC - COST OF COVARIANCE COMPUTATION')
CALL GSTATS_LABEL(209,'JBW','JBWAVGEN - COST OF NORMALIZATION BY STD')
CALL GSTATS_LABEL(210,'JBW','JBWAVGEN - COST OF POWER SPECTRUM')
CALL GSTATS_LABEL(211,'JBW','JBWAVGEN - COST OF WAVEL-SPEC TRANSF')
CALL GSTATS_LABEL(212,'JBW','JBWAVGEN - COST OF WAVEL-GRID TRANSF')
CALL GSTATS_LABEL(213,'JBW','JBWAVGEN - COST OF GATHER POWER SPECTRUM')
CALL GSTATS_LABEL(214,'JBW','JBWAVGEN - READ/TRUNC SPECTRAL STATES')
CALL GSTATS_LABEL(215,'JBW','JBWAVGEN - READ/INT GRIDPOINT STATES')
CALL GSTATS_LABEL(216,'JBW','JBWAVGEN - READ/INT PS GRIDPOINT STATES')
CALL GSTATS_LABEL(217,'JBW','JBWAVGEN - READ/INT UA GRIDPOINT STATES')
CALL GSTATS_LABEL(218,'JBW','JBWAVGEN - GRIDPOINT BALANCE')
CALL GSTATS_LABEL(219,'JBW','JBWAVGEN - SPECTRAL BALANCE')
CALL GSTATS_LABEL(220,'JBA','ALPHA HYBRID B - LINEAR')
CALL GSTATS_LABEL(221,'JBA','ALPHA HYBRID B - ADJOINT')
CALL GSTATS_LABEL(222,'JBA','ALPHA HYBRID B - SUINEP')
CALL GSTATS_LABEL(223,'JBA','ALPHA HYBRID B - SUINEP READING')
CALL GSTATS_LABEL(224,'JBA','ALPHA HYBRID B - SUINEP INTERPOLATING')
CALL GSTATS_LABEL(225,'JBA','ALPHA HYBRID B - SUINEP STORE')

!   counters 251 to 300 :  additional message passing 

CALL GSTATS_LABEL(251,'MP-','ALLGATHER    CONTROL VECTORS')
CALL GSTATS_LABEL(252,'MP-','CVSECTION    CONTROL VECTORS')
CALL GSTATS_LABEL(253,'MP-','GATHER       CONTROL VECTORS')
CALL GSTATS_LABEL(254,'MP-','SCATTER      CONTROL VECTORS')
CALL GSTATS_LABEL(255,'MP-','DOT PRODUCT  CONTROL VECTORS')
CALL GSTATS_LABEL(256,'MP-','REDUCE       CONTROL VECTORS')
CALL GSTATS_LABEL(257,'MP-','MULTISCATTER CONTROL VECTORS')
CALL GSTATS_LABEL(258,'MP-','MULTIGATHER  CONTROL VECTORS')

! couplo4 call - intefrace to oasis4
CALL GSTATS_LABEL(260,'   ','COULPLO4_DEFINITIONS in CNT4')
CALL GSTATS_LABEL(261,'   ','PRISM_DEF in COULPLO4_DEFINITIONS ')
CALL GSTATS_LABEL(262,'   ','COULPLO4_EXCHANGE in GP_MODEL  ')
CALL GSTATS_LABEL(263,'   ','PRISM_GET 3D-GP in COULPLO4_EXCHANGE ')
CALL GSTATS_LABEL(264,'   ','PRISM_PUT 3D-GP in COULPLO4_EXCHANGE ')
CALL GSTATS_LABEL(265,'   ','PRISM_PUT 2D-GP in COULPLO4_EXCHANGE ')
CALL GSTATS_LABEL(266,'   ','PRISM_PUT 3D_SP in COULPLO4_EXCHANGE ')
CALL GSTATS_LABEL(267,'   ','PRISM_PUT 2D_SP in COULPLO4_EXCHANGE ')

CALL GSTATS_LABEL(301,'   ','INTERP_OBSAD ')
CALL GSTATS_LABEL(302,'   ','INTERP_OBSAD_1 ')
CALL GSTATS_LABEL(303,'   ','OPK_OBSCOR ')
CALL GSTATS_LABEL(304,'   ','INTERP_OBSAD_2 ')
CALL GSTATS_LABEL(305,'   ','INTERP_OBSAD_3 ')
CALL GSTATS_LABEL(306,'   ','SUJBWAVALLO ')
CALL GSTATS_LABEL(307,'   ','INTERP_OBS ')
CALL GSTATS_LABEL(308,'MP-','VARBC_PRED ')
CALL GSTATS_LABEL(309,'MP-','VARBC_SETUP ')

CALL GSTATS_LABEL(400,'   ','GSTATS ')
CALL GSTATS_LABEL(401,'   ','GSTATS HOOK')

!   counters 500 to 2000
CALL GSTATS_LABEL(501,'MPL','SLCOMM2_COMMS PART1')
CALL GSTATS_LABEL(502,'MPL','SLCOMM2A_COMMS PART1')
CALL GSTATS_LABEL(507,'MPL','TRSTOM_COMMS')
CALL GSTATS_LABEL(508,'MPL','TRMTOS_COMMS')
CALL GSTATS_LABEL(509,'MPL','SLCOMM1_COMMS')
CALL GSTATS_LABEL(511,'MPL','SLCOMM2_COMMS PART2')
CALL GSTATS_LABEL(512,'MPL','SLCOMM2A_COMMS PART2')
CALL GSTATS_LABEL(513,'MPL','GOM_MESSAGE_PASSING_COMMS')
CALL GSTATS_LABEL(514,'MPL','GOM_MESSAGE_PASSING_AD_COMMS')
CALL GSTATS_LABEL(515,'MPL','STATPRED')
CALL GSTATS_LABEL(516,'MPL','SUVARBC')
CALL GSTATS_LABEL(517,'MPL','CVARBCAD')
CALL GSTATS_LABEL(518,'MPL','CVARBCINAD')
CALL GSTATS_LABEL(520,'MPL','DRESDDH ')
CALL GSTATS_LABEL(521,'MPL','STEPO  ALLREDUCE  ')
CALL GSTATS_LABEL(523,'MPL','GATHEREIGMD  ')
CALL GSTATS_LABEL(524,'MPL','SCATTER_CTLVEC ')
CALL GSTATS_LABEL(525,'MPL','MULTISCATTER_CTLVEC ')
CALL GSTATS_LABEL(528,'MPL','GATHER_CTLVEC    ')
CALL GSTATS_LABEL(529,'MPL','ALLGATHER_CTLVEC ')
CALL GSTATS_LABEL(530,'MPL','ODBMP-BCAST')
CALL GSTATS_LABEL(531,'MPL','ODBMP-ALLREDUCE')
CALL GSTATS_LABEL(532,'MPL','ODBMP-SEND')
CALL GSTATS_LABEL(533,'MPL','ODBMP-RECV')
CALL GSTATS_LABEL(534,'MPL','ODBMP-ALLGATHERV')
CALL GSTATS_LABEL(535,'MPL','SHUFFLEDB')
CALL GSTATS_LABEL(536,'MPL','MULTIGATHER_CTLVEC ')
CALL GSTATS_LABEL(537,'MPL','ALLGATHER_CVSECTION')
CALL GSTATS_LABEL(538,'MPL','HALO_EXCHANGE GP_DERS')
CALL GSTATS_LABEL(602,'MPL','BRPTOB       ')
CALL GSTATS_LABEL(604,'MPL','ALLGATHER IN GATHERGPF    ')
CALL GSTATS_LABEL(605,'MPL','GATHERTC GATH')
CALL GSTATS_LABEL(606,'MPL','GATHERTC SCAT')
CALL GSTATS_LABEL(607,'MPL','COMMSPNORM  GATH')
CALL GSTATS_LABEL(608,'MPL','COMMSPNORM1 GATH')
CALL GSTATS_LABEL(609,'MPL','GATHERBDY GATH ')
CALL GSTATS_LABEL(610,'MPL','GATHERCOSTO  ')
CALL GSTATS_LABEL(611,'MPL','GATHERCOST2  ')
CALL GSTATS_LABEL(615,'MPL','BROADCAST IN ALLGATHER_CVSECTION ')
CALL GSTATS_LABEL(616,'MPL','ALLGATHERV IN MPBCASTSCFLD ')
CALL GSTATS_LABEL(617,'MPL','BROADCAST IN MPBCASTINTFLD WAM ')
CALL GSTATS_LABEL(618,'MPL','GRFIELD WAM ')
CALL GSTATS_LABEL(619,'MPL','BROADCAST IN INMARS IN WAM ')
CALL GSTATS_LABEL(620,'MPL','BROADCAST IN MPBCASTGRDFLD WAM ')
CALL GSTATS_LABEL(621,'MPL','BROADCAST IN GETSTRESS WAM ')
CALL GSTATS_LABEL(622,'MPL','SEND/RECV IN READWIND WAM ')
CALL GSTATS_LABEL(623,'MPL','SEND/RECV IN GETSPEC WAM ')
CALL GSTATS_LABEL(624,'MPL','SEND/RECV IN MPDISTRIBSCFLD WAM ')
CALL GSTATS_LABEL(625,'MPL','SCAQC REDUCE ')
CALL GSTATS_LABEL(626,'MPL','BROADCAST GETMINI ')
CALL GSTATS_LABEL(627,'MPL','ALLGATHER GETMINI2')
CALL GSTATS_LABEL(628,'MPL','BROADCAST IOSTREAM INQUIRE ')
CALL GSTATS_LABEL(629,'MPL','GATHER CLOSE IOSTREAM  ')
CALL GSTATS_LABEL(630,'MPL','GATHER IOSTREAM_STATS  ')
CALL GSTATS_LABEL(631,'MPL','BROADCAST IOSTREAM SPEC_IN')
CALL GSTATS_LABEL(632,'MPL','BROADCAST IOSTREAM GRID_IN')
CALL GSTATS_LABEL(633,'MPL','IOSTREAM GATH_GRID_FP')
CALL GSTATS_LABEL(634,'MPL','BROADCAST IOSTREAM GRID_IN')
CALL GSTATS_LABEL(635,'MPL','SUJBWAVELET')
CALL GSTATS_LABEL(636,'MPL','SUJBWAVALLO MATRICES')
CALL GSTATS_LABEL(637,'MPL','BROADCAST SUJBBAL')
CALL GSTATS_LABEL(638,'MPL','ECSET')
CALL GSTATS_LABEL(639,'MPL','SUJBWAVALLO BCASTS')
CALL GSTATS_LABEL(640,'MPL','SUJBWAVALLO INDXL2G')
CALL GSTATS_LABEL(647,'MPL','IFSTOWAM WAM')
CALL GSTATS_LABEL(648,'MPL','IOSTREAM READ_RECORD 2 ')
CALL GSTATS_LABEL(649,'MPL','IOSTREAM WRITE_RECORD ')
CALL GSTATS_LABEL(650,'MPL','IOSTREAM READ_RECORD ')
CALL GSTATS_LABEL(651,'MPL','DVSECTION ALLGATH ')
CALL GSTATS_LABEL(652,'MPL','DOT_PROD ALLGATH ')
CALL GSTATS_LABEL(653,'MPL','SCAT_VECT BROAD ')
CALL GSTATS_LABEL(654,'MPL','GATH_VECT GATH ')
CALL GSTATS_LABEL(655,'MPL','BROADCAST IN SUARG ')
CALL GSTATS_LABEL(656,'MPL','ALLGATHERV IN DOT_PRODUCT_CTLVEC')
CALL GSTATS_LABEL(657,'MPL','ALLGATHERV IN DOT_PRODUCT_CTLVEC')
CALL GSTATS_LABEL(658,'MPL','SIGCHECK            ')
CALL GSTATS_LABEL(659,'MPL','ALLTOALL IN WRITE_SPEC     ')
CALL GSTATS_LABEL(660,'MPL','BROADCAST IN INIFGER       ')
CALL GSTATS_LABEL(661,'MPL','BROADCAST IN SUINSKTE       ')
CALL GSTATS_LABEL(663,'MPL','BROADCAST IN COMMJBDAT ')
CALL GSTATS_LABEL(664,'MPL','BROADCAST IN COMMJBBAL ')
CALL GSTATS_LABEL(665,'MPL','BROADCAST IN COMMFCE2  ')
CALL GSTATS_LABEL(666,'MPL','SEND and RECV IN GOM_PRINT_RMS ')
CALL GSTATS_LABEL(667,'MPL','BROADCAST IN SUECRAD ')
CALL GSTATS_LABEL(668,'MPL','GATHERCOST1  ')
CALL GSTATS_LABEL(669,'MPL','ALLREDUCE IN SUPRFFCE ')
CALL GSTATS_LABEL(670,'MPL','SCATTERV IN READ_SPEC ')
CALL GSTATS_LABEL(671,'MPL','DOT PRODUCT SPECTRAL FIELDS')
CALL GSTATS_LABEL(672,'MPL','WAMODEL MPGATHERGRDFLD    ')
CALL GSTATS_LABEL(673,'MPL','WAMODEL MPGATHERSPP       ')
CALL GSTATS_LABEL(674,'MPL','WAMODEL MPGATHERSCFLD     ')
CALL GSTATS_LABEL(675,'MPL','WAMODEL WAMNORM           ')
CALL GSTATS_LABEL(676,'MPL','WAMODEL MPEXCHNG          ')
CALL GSTATS_LABEL(677,'MPL','WAMODEL MPGATHERFL        ')
CALL GSTATS_LABEL(678,'MPL','WVXF2GB ')
CALL GSTATS_LABEL(679,'MPL','SUSPECG SEND 2         ')
CALL GSTATS_LABEL(680,'MPL','SUSPECG SEND 3         ')
CALL GSTATS_LABEL(681,'MPL','SUSPECG 4              ')
CALL GSTATS_LABEL(682,'MPL','SUGRIDG SEND           ')
CALL GSTATS_LABEL(683,'MPL','SUGRIDG RECV           ')
CALL GSTATS_LABEL(685,'MPL','SUGRIDUG 1          ')
CALL GSTATS_LABEL(686,'MPL','WAM MPFLDTOIFS      ')
CALL GSTATS_LABEL(687,'MPL','SUGRIDUG 2          ')
CALL GSTATS_LABEL(688,'MPL','WROUTGPGB           ')
CALL GSTATS_LABEL(689,'MPL','WROUTSPGB           ')
CALL GSTATS_LABEL(690,'MPL','ALLTOALLV IN READ_SPEC ')
CALL GSTATS_LABEL(691,'MPL','MKGLOBSTAB ')
CALL GSTATS_LABEL(692,'MPL','OUTWSPEC WAM')
CALL GSTATS_LABEL(693,'MPL','OUTGRID WAM')
CALL GSTATS_LABEL(694,'MPL','MPDECOMP WAM ')
CALL GSTATS_LABEL(695,'MPL','SARINVERT GATHER WAM ')
CALL GSTATS_LABEL(696,'MPL','SARINVERT BCAST WAM')

! barrier counter 700 ---> 800

CALL GSTATS_LABEL(701,'GBR','GBAR IN TRMTOS          ')
CALL GSTATS_LABEL(702,'GBR','GBAR TO TIME BARRIER CALL')
CALL GSTATS_LABEL(703,'GBR','GBAR IN TRSTOM          ')
CALL GSTATS_LABEL(704,'BAR','BARRIER IN GP_MODEL        ')
CALL GSTATS_LABEL(705,'GBR','GBAR IN MULTISCATTER_CTLVEC')
CALL GSTATS_LABEL(706,'BAR','BARRIER IN COMMJBDAT       ')
CALL GSTATS_LABEL(707,'GBR','GBAR IN IOSTREAM_MIX:IO_INQUIRE ')
CALL GSTATS_LABEL(708,'GBR','GBAR IN RDVARBC         ')
CALL GSTATS_LABEL(709,'GBR','GBAR IN GOM_MESSAGE_PASSING')
CALL GSTATS_LABEL(710,'GBR','GBAR IN GOM_MESSAGE_PASING_AD')
CALL GSTATS_LABEL(711,'GBR','GBAR IOSTREAM SPEC_OUT  ')
CALL GSTATS_LABEL(712,'GBR','GBAR IN GATHERCOSTO ')
CALL GSTATS_LABEL(713,'GBR','GBAR IN SUGRIDG         ')
CALL GSTATS_LABEL(714,'GBR','GBAR IN SCATTER_CTLVEC')
CALL GSTATS_LABEL(715,'GBR','GBAR IN SUSPECG         ')
CALL GSTATS_LABEL(716,'GBR','GBAR IN GATHER_CTLVEC')
CALL GSTATS_LABEL(718,'BAR','BARRIER IN SIGCHECK        ')
CALL GSTATS_LABEL(719,'BAR','BARRIER IN OUTBS           ')
CALL GSTATS_LABEL(720,'GBR','GBAR IN ALLGATHER_CTLVEC')
CALL GSTATS_LABEL(721,'BAR','BARRIER IN SUJBSTD         ')
CALL GSTATS_LABEL(722,'GBR','GBAR IN ALLGATHER_CVSECTION')
CALL GSTATS_LABEL(723,'GBR','GBAR IN SUJBWAVALLO')
CALL GSTATS_LABEL(724,'GBR','GBAR IN MULTIGATHER_CTLVEC')
CALL GSTATS_LABEL(725,'GBR','GBAR IN EVCOST         ')
CALL GSTATS_LABEL(726,'GBR','GBAR IN IOSTREAM_MIX:SPEC_IN')
CALL GSTATS_LABEL(727,'GBR','GBAR IN IOSTREAM_MIX:GRID_IN')
CALL GSTATS_LABEL(728,'GBR','GBAR IN IOSTREAM_MIX:GATH_GRID_FP')
CALL GSTATS_LABEL(729,'GBR','GBAR IN IOSTREAM_MIX:IOSTREAM_STATS')
CALL GSTATS_LABEL(730,'GBR','BARRIER IN EC_PHYS         ')
CALL GSTATS_LABEL(731,'GBR','BARRIER IN EC_PHYS_TL      ')
CALL GSTATS_LABEL(732,'GBR','BARRIER IN EC_PHYS_AD      ')
CALL GSTATS_LABEL(733,'GBR','BARRIER IN WVXF2GB         ')
CALL GSTATS_LABEL(734,'GBR','BARRIER IN MPFLDTOIFS      ')
CALL GSTATS_LABEL(736,'GBR','BARRIER IN MPEXCHNG        ')
CALL GSTATS_LABEL(737,'BAR','BARRIER IN CNT4            ')
CALL GSTATS_LABEL(738,'GBR','GBAR IN IOSTREAM WRITE_RECORD      ')
CALL GSTATS_LABEL(739,'GBR','GBAR IN IOSTREAM READ_RECORD      ')
CALL GSTATS_LABEL(740,'GBR','GBAR IN WROUTGPGB       ')
CALL GSTATS_LABEL(741,'GBR','GBAR IN WROUTSPGB       ')
CALL GSTATS_LABEL(742,'GBR','GBAR IN SUJBWAVALLO MAT')
CALL GSTATS_LABEL(745,'GBR','GBAR IN IOSTREAM CLOSE     ')
CALL GSTATS_LABEL(746,'GBR','GBAR IN DOT_PRODUCT_CTLVEC ')
CALL GSTATS_LABEL(747,'GBR','GBAR IN DOT_PRODUCT_CTLVEC ')
CALL GSTATS_LABEL(748,'GBR','GBAR IN SLCOMM2 PART2 ')
CALL GSTATS_LABEL(749,'GBR','GBAR IN SLCOMM2A PART2 ')
CALL GSTATS_LABEL(750,'BAR','BARRIER IN OUTGRID WAM     ')
CALL GSTATS_LABEL(752,'BAR','BARRIER IN WAMASSI         ')
CALL GSTATS_LABEL(753,'BAR','BARRIER IN WAMODEL         ')
CALL GSTATS_LABEL(754,'GBR','GBAR IN TASKOB          ')
CALL GSTATS_LABEL(755,'GBR','GBAR IN TASKOBTL        ')
CALL GSTATS_LABEL(756,'GBR','GBAR IN TASKOBAD        ')
CALL GSTATS_LABEL(757,'GBR','GBAR IN SLCOMM1         ')
CALL GSTATS_LABEL(758,'GBR','GBAR IN SLCOMM2A        ')
CALL GSTATS_LABEL(759,'GBR','GBAR IN SLCOMM2 PART1   ')
CALL GSTATS_LABEL(760,'GBR','GBAR IN SLCOMM          ')
CALL GSTATS_LABEL(761,'GBR','GBAR IN TRGTOL          ')
CALL GSTATS_LABEL(762,'GBR','GBAR IN TRLTOG          ')
CALL GSTATS_LABEL(763,'GBR','GBAR IN TRLTOM          ')
CALL GSTATS_LABEL(764,'GBR','GBAR IN TRMTOL          ')
CALL GSTATS_LABEL(765,'GBR','GBAR IN TRMTOS          ')
CALL GSTATS_LABEL(766,'GBR','GBAR IN TRSTOM          ')
CALL GSTATS_LABEL(767,'GBR','GBAR IN GOM_MESSAGE_PASSING')
CALL GSTATS_LABEL(768,'GBR','GBAR IN GOM_MESSAGE_PASSING_AD')
CALL GSTATS_LABEL(769,'GBR','GBAR IN GATHERGPF       ')
CALL GSTATS_LABEL(770,'GBR','GBAR IN GATHERTC        ')
CALL GSTATS_LABEL(771,'GBR','GBAR IN GATHERBDY       ')
CALL GSTATS_LABEL(773,'GBR','GBAR IN COMMSPNORM      ')
CALL GSTATS_LABEL(774,'GBR','GBAR IN COMMJBDAT       ')
CALL GSTATS_LABEL(775,'GBR','GBAR IN COMMJBBAL       ')
CALL GSTATS_LABEL(776,'GBR','GBAR IN COMMFCE2        ')
CALL GSTATS_LABEL(777,'GBR','GBAR IN GATHERCOST1     ')
CALL GSTATS_LABEL(778,'GBR','GBAR IN GATHERCOST2     ')
CALL GSTATS_LABEL(779,'GBR','GBAR IN MKGLOBSTAB      ')
CALL GSTATS_LABEL(780,'GBR','GBAR IN BRPTOB          ')
CALL GSTATS_LABEL(781,'GBR','GBAR IN DOT_PRODUCT_CTLVEC')
CALL GSTATS_LABEL(782,'GBR','GBAR IN ODBMP-ALLREDUCE')
CALL GSTATS_LABEL(783,'BAR','BARRIER IN SUTRLE')
CALL GSTATS_LABEL(784,'BAR','BARRIER IN GATH_GRID_CTL')
CALL GSTATS_LABEL(785,'BAR','BARRIER IN GATH_SPEC_CONTROL')
CALL GSTATS_LABEL(786,'BAR','BARRIER IN DIST_GRID_CTL')
CALL GSTATS_LABEL(787,'BAR','BARRIER IN DIST_SPEC_CONTROL')
CALL GSTATS_LABEL(788,'GBR','GBAR IN GATH_SPEC_CONTROL')
CALL GSTATS_LABEL(789,'GBR','GBAR IN GATH_GRID_CTL')
CALL GSTATS_LABEL(790,'GBR','GBAR IN DIST_SPEC_CONTROL')
CALL GSTATS_LABEL(791,'GBR','GBAR IN DIST_GRID_CTL')
CALL GSTATS_LABEL(792,'BAR','BARRIER IN WRMLFP')
CALL GSTATS_LABEL(793,'BAR','BARRIER IN WRRESF')
CALL GSTATS_LABEL(794,'BAR','BARRIER IN DRESDDH')
CALL GSTATS_LABEL(795,'GBR','BARRIER IN SUSTAONL')
CALL GSTATS_LABEL(796,'GBR','BARRIER IN IFSTOWAM')
CALL GSTATS_LABEL(797,'GBR','GBAR IN OBSHORAD')
CALL GSTATS_LABEL(798,'BAR','BARRIER IN SULEG')
CALL GSTATS_LABEL(799,'BAR','BARRIER IN POSDDH')

!   counters 800 to 900 :  trans package    

CALL GSTATS_LABEL(801,'MPL','MPI IN SUTRLE_MOD ')
CALL GSTATS_LABEL(803,'MPL','TRGTOL_COMMS')
CALL GSTATS_LABEL(804,'MPL','TRGTOL_COMMS (GPNORM)')
CALL GSTATS_LABEL(805,'MPL','TRLTOG_COMMS')
CALL GSTATS_LABEL(806,'MPL','TRLTOM_COMMS')
CALL GSTATS_LABEL(807,'MPL','TRMTOL_COMMS')
CALL GSTATS_LABEL(809,'MPL','GATH_GRID_CTL_COMMS')
CALL GSTATS_LABEL(810,'MPL','GATH_SPEC_CONTROL_COMMS')
CALL GSTATS_LABEL(811,'MPL','DIST_GRID_CTL_COMMS')
CALL GSTATS_LABEL(812,'MPL','DIST_SPEC_CONTROL_COMMS')
CALL GSTATS_LABEL(813,'MPL','EVCOST')
CALL GSTATS_LABEL(814,'MPL','TRANS SUSTAONL')
CALL GSTATS_LABEL(815,'MPL','GPNORM_TRANS')
CALL GSTATS_LABEL(816,'MPL','GPNORM_TRANS')
CALL GSTATS_LABEL(817,'MPL','SUOBSCOR')
CALL GSTATS_LABEL(851,'MPL','SULEG - SUPOLF')
CALL GSTATS_LABEL(852,'MPL','SULEG - CONSTRUCT_BUTTERFLY')

CALL GSTATS_LABEL(901,'MPL','MWAVE_PUT_TL')
CALL GSTATS_LABEL(902,'MPL','MWAVE_NEAREST')
CALL GSTATS_LABEL(903,'MPL','MWAVE_GP2OBS')
CALL GSTATS_LABEL(904,'MPL','MWAVE_OBS2GP')
CALL GSTATS_LABEL(905,'MPL','MWAVE_IGP2OBS')
CALL GSTATS_LABEL(906,'MPL','MWAVE_IOBS2GP')
CALL GSTATS_LABEL(907,'MPL','MWAVE_PUT')
CALL GSTATS_LABEL(908,'BAR','BARRIER IN MWAVE_NEAREST')
CALL GSTATS_LABEL(909,'BAR','BARRIER IN MWAVE_GP2OBS')
CALL GSTATS_LABEL(910,'BAR','BARRIER IN MWAVE_OBS2GP')
CALL GSTATS_LABEL(911,'BAR','BARRIER IN MWAVE_IGP2OBS')
CALL GSTATS_LABEL(912,'BAR','BARRIER IN MWAVE_IOBS2GP')

CALL GSTATS_LABEL(920,'MPL','BCAST IN BCASTCOV')
CALL GSTATS_LABEL(921,'GBR','GBAR IN BCASTCOV')

! OMP section
! oct-03 the following OMP gstats statements has not been checked
! 1014, 1015, 1045, 1053, 1072, 1078, 1080, 1082, 1134, 1170
! 1201, 1214, 1215, 1219, 1412, 1424
! 1607, 1608, 1610, 1611, 1543, 1644, 1651

CALL GSTATS_LABEL(1001,'OMP','PHYSICS      ')
CALL GSTATS_LABEL(1002,'OMP','PHYSICS CLDPP T/S')
CALL GSTATS_LABEL(1004,'OMP','CALL_SL     1')
CALL GSTATS_LABEL(1005,'OMP','CALL_SL     2')
CALL GSTATS_LABEL(1006,'OMP','GP_MODEL_TL 1')
CALL GSTATS_LABEL(1007,'OMP','GP_MODEL_TL 2')
CALL GSTATS_LABEL(1008,'OMP','GP_MODEL_TL 3')
CALL GSTATS_LABEL(1009,'OMP','CALL_SL_TL  1')
CALL GSTATS_LABEL(1010,'OMP','CALL_SL_TL  2')
CALL GSTATS_LABEL(1011,'OMP','GP_MODEL_AD 1')
CALL GSTATS_LABEL(1012,'OMP','GP_MODEL_AD 2')
CALL GSTATS_LABEL(1013,'OMP','EC_PHYS_DRV_AD 3')
CALL GSTATS_LABEL(1014,'OMP','GP_MODEL_AD 4')
CALL GSTATS_LABEL(1015,'OMP','GP_MODEL_AD 5')
CALL GSTATS_LABEL(1016,'OMP','GP_MODEL_AD 6')
CALL GSTATS_LABEL(1017,'OMP','CALL_SL_AD  1')
CALL GSTATS_LABEL(1018,'OMP','CALL_SL_AD  2')
CALL GSTATS_LABEL(1019,'OMP','MKGLOBSTAB ')
CALL GSTATS_LABEL(1020,'OMP','SLINT LAID*IOBS ')
CALL GSTATS_LABEL(1021,'OMP','LAIDLIC ')
CALL GSTATS_LABEL(1022,'OMP','SPCM        1')
CALL GSTATS_LABEL(1023,'OMP','SPCM        2')
CALL GSTATS_LABEL(1024,'OMP','SPCM        3')
CALL GSTATS_LABEL(1025,'OMP','CPG          ')
CALL GSTATS_LABEL(1028,'OMP','SPCIMPFSOLVE ')
CALL GSTATS_LABEL(1029,'OMP','SPCMAD      1')
CALL GSTATS_LABEL(1030,'OMP','SPCMAD      2')
CALL GSTATS_LABEL(1031,'OMP','SPCMAD      3')
CALL GSTATS_LABEL(1032,'OMP','SCAN2M GFL ')
CALL GSTATS_LABEL(1033,'OMP','SCAN2M    2')
CALL GSTATS_LABEL(1034,'OMP','SCAN2M COBS ')
CALL GSTATS_LABEL(1035,'OMP','SPCHORAD     ')
CALL GSTATS_LABEL(1036,'OMP','TRANSDIR_MDL ')
CALL GSTATS_LABEL(1037,'OMP','SPCHOR       ')
CALL GSTATS_LABEL(1038,'OMP','SPECRT GPRCP')
CALL GSTATS_LABEL(1039,'OMP','SPCIMPFSOLVEAD')
CALL GSTATS_LABEL(1040,'OMP','GP_MODEL CPGLAG')
CALL GSTATS_LABEL(1041,'OMP','SCAN2MTL    1')
CALL GSTATS_LABEL(1042,'OMP','SCAN2MAD    1')
CALL GSTATS_LABEL(1043,'OMP','SCAN2MAD    2')
CALL GSTATS_LABEL(1044,'OMP','SCAN2MAD    3')
CALL GSTATS_LABEL(1045,'OMP','SCAN2MAD    4')
CALL GSTATS_LABEL(1046,'OMP','SCAN2MTL    2')
CALL GSTATS_LABEL(1047,'OMP','SCAN2MAD    5')
CALL GSTATS_LABEL(1048,'OMP','SPNORMBM    1')
CALL GSTATS_LABEL(1049,'OMP','CVARGPAD     ')
CALL GSTATS_LABEL(1050,'OMP','TRAJ_MAIN_MOD   1')
CALL GSTATS_LABEL(1051,'OMP','TRAJ_MAIN_MOD   2')
CALL GSTATS_LABEL(1052,'OMP','TRAJ_MAIN_MOD   3')
CALL GSTATS_LABEL(1053,'OMP','TRAJ_MAIN_MOD   4')
CALL GSTATS_LABEL(1054,'OMP','PACK_LOC_MS 1')
CALL GSTATS_LABEL(1055,'OMP','READ_GRID_TRAJ 1')
CALL GSTATS_LABEL(1056,'OMP','READ_GRID_TRAJ 2')
CALL GSTATS_LABEL(1057,'OMP','READ_TRAJ_SPEC 1')
CALL GSTATS_LABEL(1058,'OMP','READ_TRAJ_SPEC 2')
CALL GSTATS_LABEL(1059,'OMP','READ_TRAJ_SPEC 3')
CALL GSTATS_LABEL(1060,'OMP','TASKOB      1')
CALL GSTATS_LABEL(1061,'OMP','TASKOBTL    1')
CALL GSTATS_LABEL(1062,'OMP','TASKOBAD    1')
CALL GSTATS_LABEL(1063,'OMP','GPMKTEND     ')
CALL GSTATS_LABEL(1064,'OMP','CPQTUV       ')
CALL GSTATS_LABEL(1065,'OMP','IN-COAPHY    ')
CALL GSTATS_LABEL(1066,'OMP','OUT-COAPHY   ')
CALL GSTATS_LABEL(1067,'OMP','TASKOBAD INIT')
CALL GSTATS_LABEL(1068,'OMP','TASKOB   INIT')
CALL GSTATS_LABEL(1082,'OMP','SCREEN       ')
CALL GSTATS_LABEL(1085,'OMP','LAIDLIOBSAD  ')
CALL GSTATS_LABEL(1090,'OMP','BALVERT      ')
CALL GSTATS_LABEL(1092,'OMP','BALVERTAD    ')
CALL GSTATS_LABEL(1094,'OMP','DIGFIL      ')
CALL GSTATS_LABEL(1096,'OMP','JGHCORI      ')
CALL GSTATS_LABEL(1097,'OMP','JGVCORI      ')
CALL GSTATS_LABEL(1098,'OMP','NABLA  GP_DERS')
CALL GSTATS_LABEL(1099,'OMP','COPY 1 GP_DERS')
CALL GSTATS_LABEL(1100,'OMP','COPY 2 GP_DERS')
CALL GSTATS_LABEL(1109,'OMP','TRSTOM      1')
CALL GSTATS_LABEL(1110,'OMP','TRSTOM      2')
CALL GSTATS_LABEL(1111,'OMP','TRSTOM      3')
CALL GSTATS_LABEL(1112,'OMP','TRMTOS      1')
CALL GSTATS_LABEL(1113,'OMP','TRMTOS      2')
CALL GSTATS_LABEL(1114,'OMP','TRMTOS      3')
CALL GSTATS_LABEL(1115,'OMP','SLCOMM      1')
CALL GSTATS_LABEL(1116,'OMP','SLCOMM      2')
CALL GSTATS_LABEL(1118,'OMP','SLCOMM2a    1')
CALL GSTATS_LABEL(1119,'OMP','SLCOMM1     1')
CALL GSTATS_LABEL(1120,'OMP','SLCOMM1     2')
CALL GSTATS_LABEL(1121,'OMP','SLCOMM2a    2')
CALL GSTATS_LABEL(1132,'OMP','SLEXTPOLAD   ')
CALL GSTATS_LABEL(1133,'OMP','SLEXTPOL     ')
CALL GSTATS_LABEL(1135,'OMP','GOM_MSG_PASS_AD')
CALL GSTATS_LABEL(1136,'OMP','GOM_CONSTRUCT_OBS')
CALL GSTATS_LABEL(1140,'OMP','DRESDDH      ')
CALL GSTATS_LABEL(1141,'OMP','SUTRLE       ')
CALL GSTATS_LABEL(1170,'OMP','DIST_VECTOR 1')

CALL GSTATS_LABEL(1201,'OMP','RADDRV      1')
CALL GSTATS_LABEL(1202,'OMP','RADDRV      2')
CALL GSTATS_LABEL(1203,'OMP','RADDRV      3')
CALL GSTATS_LABEL(1205,'OMP','RADINTG-INPUT LOOP1')
CALL GSTATS_LABEL(1206,'OMP','RADINTG-INPUT LOOP2')
CALL GSTATS_LABEL(1207,'OMP','RADINTG-INPUT LOOP3')
CALL GSTATS_LABEL(1208,'OMP','RADINTG-INPUT LOOP4')
CALL GSTATS_LABEL(1209,'OMP','RADINTG-INPUT LOOP5')
CALL GSTATS_LABEL(1210,'OMP','RADINTG-RADLSW')
CALL GSTATS_LABEL(1211,'OMP','RADINTG-OUTPUT LOOP1')
CALL GSTATS_LABEL(1212,'OMP','RADINTG-OUTPUT LOOP2')
CALL GSTATS_LABEL(1213,'OMP','RADINTG-OUTPUT LOOP3')
CALL GSTATS_LABEL(1214,'OMP','COPY_SPEC_2SPA 1')
CALL GSTATS_LABEL(1215,'OMP','COPY_SPEC_2SPA 2')
CALL GSTATS_LABEL(1216,'OMP','GPNORM2')
CALL GSTATS_LABEL(1217,'OMP','SUALSPA 1')
CALL GSTATS_LABEL(1219,'OMP','SUALSPA 3')
CALL GSTATS_LABEL(1220,'OMP','SL2_PACK')
CALL GSTATS_LABEL(1221,'OMP','SL2_UNPACK')
CALL GSTATS_LABEL(1222,'OMP','GP_MODEL_AD INIT ')
CALL GSTATS_LABEL(1223,'OMP','GPNORM3')
CALL GSTATS_LABEL(1224,'OMP','COPYGOM5T0 ')
CALL GSTATS_LABEL(1225,'OMP','BALNONLINTL ')
CALL GSTATS_LABEL(1226,'OMP','BALNONLINAD ')
CALL GSTATS_LABEL(1227,'OMP','GP_MODEL_TL INIT ')
CALL GSTATS_LABEL(1228,'OMP','GP_MODEL INIT ')
CALL GSTATS_LABEL(1230,'OMP','PRECOND ')
CALL GSTATS_LABEL(1231,'OMP','CONGRAD OMP')
CALL GSTATS_LABEL(1232,'OMP','DOT_PRODUCT_CTLVEC OMP 1')
CALL GSTATS_LABEL(1233,'OMP','CONGRAD OMP 1')
CALL GSTATS_LABEL(1234,'OMP','DOT_PRODUCT_CTLVEC OMP 2')
CALL GSTATS_LABEL(1235,'OMP','PREPPCM ')
CALL GSTATS_LABEL(1236,'OMP','WAM OMP ')
CALL GSTATS_LABEL(1237,'OMP','MKCMARPL OMP')
CALL GSTATS_LABEL(1238,'OMP','XFORMEV')
CALL GSTATS_LABEL(1239,'OMP','WREVECS')

CALL GSTATS_LABEL(1251,'OMP','SULEG - SUPOLF')
CALL GSTATS_LABEL(1252,'OMP','SULEG - CONSTRUCT_BUTTERFLY')
CALL GSTATS_LABEL(1253,'OMP','CONSTRUCT BUTTERFLY ')

CALL GSTATS_LABEL(1401,'OMP','SETUP GAWLM')
CALL GSTATS_LABEL(1403,'OMP','SETUP MODGRIN 2')
CALL GSTATS_LABEL(1404,'OMP','SETUP MODGRIN 3')
CALL GSTATS_LABEL(1405,'OMP','SETUP SUGEM2 1')
CALL GSTATS_LABEL(1406,'OMP','SETUP SUGEM2 2')
CALL GSTATS_LABEL(1407,'OMP','SETUP SUGRIDF 1')
CALL GSTATS_LABEL(1408,'OMP','SETUP SUGRIDF 2')
CALL GSTATS_LABEL(1409,'OMP','SETUP SUGRIDG X')
CALL GSTATS_LABEL(1411,'OMP','SETUP SUGRIDUG 2')
CALL GSTATS_LABEL(1412,'OMP','SETUP SURAND2')
CALL GSTATS_LABEL(1424,'OMP','SETUP SUSPECG 2')
CALL GSTATS_LABEL(1425,'OMP','WVXF2GB ')
CALL GSTATS_LABEL(1426,'OMP','WVRG2XF ')
CALL GSTATS_LABEL(1427,'OMP','GPNORM1 ')
CALL GSTATS_LABEL(1428,'OMP','GATHERGPF ')
CALL GSTATS_LABEL(1429,'OMP','GPNORM_TRANS ')
CALL GSTATS_LABEL(1430,'OMP','WAMODEL 1')
CALL GSTATS_LABEL(1431,'OMP','WAMODEL 2')
CALL GSTATS_LABEL(1432,'OMP','WAMODEL 3')
CALL GSTATS_LABEL(1433,'OMP','SPEC2FDB WAM ')
CALL GSTATS_LABEL(1434,'OMP','SCAN2M HPOS ')
CALL GSTATS_LABEL(1435,'OMP','SCAN2M VPOS ')
CALL GSTATS_LABEL(1436,'OMP','IFSTOWAM 1 ')
CALL GSTATS_LABEL(1437,'OMP','IFSTOWAM 2 ')
CALL GSTATS_LABEL(1438,'OMP','WRITESD ')
CALL GSTATS_LABEL(1439,'OMP','WAMODEL 4')
CALL GSTATS_LABEL(1440,'OMP','GP_NEAREST')
CALL GSTATS_LABEL(1441,'OMP','JBVCOR_WAVELETIN ')
CALL GSTATS_LABEL(1442,'OMP','JBVCOR_WAVELETINAD ')
CALL GSTATS_LABEL(1443,'OMP','WAVXFORM ')
CALL GSTATS_LABEL(1444,'OMP','BGVECS ')
CALL GSTATS_LABEL(1445,'OMP','CVAR2IN ')
CALL GSTATS_LABEL(1446,'OMP','CVARBCIN ')
CALL GSTATS_LABEL(1447,'OMP','CVARBCINAD ')
CALL GSTATS_LABEL(1448,'OMP','CVAR2INAD ')
CALL GSTATS_LABEL(1450,'OMP','TRANSDIR_WAVELET ')
CALL GSTATS_LABEL(1451,'OMP','TRANSDIR_WAVELETAD ')
CALL GSTATS_LABEL(1452,'OMP','TRANSINV_WAVELET ')
CALL GSTATS_LABEL(1453,'OMP','TRANSINV_WAVELETAD ')
CALL GSTATS_LABEL(1454,'OMP','JBTOMODEL ')
CALL GSTATS_LABEL(1455,'OMP','JBTOMODELAD ')
CALL GSTATS_LABEL(1456,'OMP','SUJQCOR ')
CALL GSTATS_LABEL(1460,'OMP','assign_cv_cv ')
CALL GSTATS_LABEL(1461,'OMP','CVARGPTL ')
CALL GSTATS_LABEL(1465,'OMP','STORE_TRAJ_MAIN')
CALL GSTATS_LABEL(1470,'OMP','GRID_BICONSERV ')
CALL GSTATS_LABEL(1471,'OMP','GRID_BICUBIC ')
CALL GSTATS_LABEL(1472,'OMP','GRID_BILINEAR ')
CALL GSTATS_LABEL(1475,'OMP','READ_SPEC ')
CALL GSTATS_LABEL(1476,'OMP','WRITE_SPEC ')
CALL GSTATS_LABEL(1477,'OMP','IOSTREAM_MIX: COPYGP_GLOB2LOC ')
CALL GSTATS_LABEL(1478,'OMP','IOSTREAM_MIX: COPYGP_LOC2GLOB ')
CALL GSTATS_LABEL(1479,'OMP','IOSTREAM_MIX: SPEC_OUT ')
CALL GSTATS_LABEL(1480,'OMP','IOSTREAM_MIX: SPEC_IN ')
CALL GSTATS_LABEL(1481,'OMP','IOSTREAM_MIX: GRID_IN ')
CALL GSTATS_LABEL(1482,'OMP','IOSTREAM_MIX: GATH_GRID_FP ')
CALL GSTATS_LABEL(1483,'OMP','IOSTREAM_MIX: WRITE_RECORD ')
CALL GSTATS_LABEL(1484,'OMP','IOSTREAM_MIX: GRIB_API ENC')
CALL GSTATS_LABEL(1490,'OMP','MWAVE_NEAREST')
CALL GSTATS_LABEL(1491,'OMP','WAM CLOSEND')
CALL GSTATS_LABEL(1492,'OMP','WAM NEWWIND')
CALL GSTATS_LABEL(1493,'OMP','WAM NOTIM')
CALL GSTATS_LABEL(1494,'OMP','WAM TIMIN')
CALL GSTATS_LABEL(1495,'OMP','WAM WGRIBOUT')
CALL GSTATS_LABEL(1496,'OMP','WAM OUTWSPEC')
CALL GSTATS_LABEL(1497,'OMP','WAM MPDECOMP')
CALL GSTATS_LABEL(1501,'OMP','WAM INIT_FIELDG')
CALL GSTATS_LABEL(1502,'OMP','WAM OUTBS')
CALL GSTATS_LABEL(1503,'OMP','WAM MPFLDTOIFS')
CALL GSTATS_LABEL(1504,'OMP','WAM INITDPTHFLDS')

!   counters 1600 to 1700 :  trans package  for OMP

CALL GSTATS_LABEL(1601,'OMP','TRGTOL LOCAL ')
CALL GSTATS_LABEL(1602,'OMP','TRGTOL PACK  ')
CALL GSTATS_LABEL(1603,'OMP','TRGTOL UNPACK')
CALL GSTATS_LABEL(1604,'OMP','TRLTOG LOCAL ')
CALL GSTATS_LABEL(1605,'OMP','TRLTOG PACK  ')
CALL GSTATS_LABEL(1606,'OMP','TRLTOG UNPACK')
CALL GSTATS_LABEL(1607,'OMP','TRLTOM       ')
CALL GSTATS_LABEL(1608,'OMP','TRMTOL       ')
CALL GSTATS_LABEL(1639,'OMP','FTINV_CTL    ')
CALL GSTATS_LABEL(1640,'OMP','FTDIR_CTL    ')
CALL GSTATS_LABEL(1641,'OMP','FTINV_CTLAD  ')
CALL GSTATS_LABEL(1642,'OMP','FTDIR_CTLAD  ')
CALL GSTATS_LABEL(1643,'OMP','GATH_GRID_CTL ')
CALL GSTATS_LABEL(1644,'OMP','GATH_SPEC_CONTROL ')
CALL GSTATS_LABEL(1645,'OMP','LTDIR_CTL   - DIRECT LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(1646,'OMP','LTDIR_CTLAD - ADJ. DIRECT LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(1647,'OMP','LTINV_CTL   - INVERSE LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(1648,'OMP','LTINV_CTLAD - ADJ. INVERSE LEGENDRE TRANSFORM')
CALL GSTATS_LABEL(1650,'OMP','SUGAW_MOD    ')
CALL GSTATS_LABEL(1651,'OMP','SPNORMD      ')
CALL GSTATS_LABEL(1655,'OMP','SPCSI LDONEM=F SIGAM')
CALL GSTATS_LABEL(1656,'OMP','SPCSI LDONEM=F')
CALL GSTATS_LABEL(1657,'OMP','SPCSI LDONEM=F SITNU')
CALL GSTATS_LABEL(1658,'OMP','SPCSIAD LDONEM=F SIGAMAD')
CALL GSTATS_LABEL(1659,'OMP','SPCSIAD LDONEM=F')
CALL GSTATS_LABEL(1660,'OMP','SPCSI LDONEM=F MXMAOP')
CALL GSTATS_LABEL(1661,'OMP','WVCOUPLE Triple loop')
CALL GSTATS_LABEL(1662,'OMP','SPCSIAD LDONEM=F SITNUAD')
CALL GSTATS_LABEL(1663,'OMP','DIST_GRID_CTL ')
CALL GSTATS_LABEL(1664,'OMP','SPCSIAD LDONEM=F MXMAOP')

!SERIAL PART

! IO counter 1700 ---> 1800
CALL GSTATS_LABEL(1703,'IO-','WAM GRIB2WGRID')
CALL GSTATS_LABEL(1704,'IO-','WAM OUTINT')
CALL GSTATS_LABEL(1705,'IO-','SUJBDAT ')
CALL GSTATS_LABEL(1706,'IO-','SUJBBAL ')
CALL GSTATS_LABEL(1709,'IO-','WGRIBOUT WAM')
CALL GSTATS_LABEL(1710,'IO-','CNT4 IFLUSHFDB')
CALL GSTATS_LABEL(1711,'IO-','WRVARBC')
CALL GSTATS_LABEL(1712,'IO-','RDVARBC')
CALL GSTATS_LABEL(1713,'IO-','WRITESD FDB ')
CALL GSTATS_LABEL(1714,'SER','WVXF2GB')
CALL GSTATS_LABEL(1737,'IO-','SUOBS GET_RS_T_BIAS')
CALL GSTATS_LABEL(1739,'IO-','DB IN OBSHOR ')
CALL GSTATS_LABEL(1740,'IO-','READWIND WAM ')
CALL GSTATS_LABEL(1741,'IO-','DB IN SUOBSCOR ')
CALL GSTATS_LABEL(1742,'SER','PREGPFPOS ')
CALL GSTATS_LABEL(1752,'IO-','OPEN IN IOSTREAM ')
CALL GSTATS_LABEL(1753,'IO-','CLOSE IN IOSTREAM ')
CALL GSTATS_LABEL(1761,'SER','ECSET ')
CALL GSTATS_LABEL(1764,'IO-','WRITE IN IOSTREAM ')
CALL GSTATS_LABEL(1765,'IO-','READ IN IOSTREAM ')
CALL GSTATS_LABEL(1766,'IO-','RTSETUP I/O')
CALL GSTATS_LABEL(1767,'IO-','DB IN RD_OBS_BOXES ')
CALL GSTATS_LABEL(1770,'IO-','READT WAM I/O ')
CALL GSTATS_LABEL(1771,'IO-','MPDECOMP WAM I/O ')
CALL GSTATS_LABEL(1773,'IO-','SHUFFLE_ODB I/O ')
CALL GSTATS_LABEL(1774,'IO-','READOBA distribute_odb ')
CALL GSTATS_LABEL(1775,'IO-','WRITEOBA revmatchupdb ')
CALL GSTATS_LABEL(1776,'IO-','PRESPFPOS ')
CALL GSTATS_LABEL(1777,'IO-','READOBA matchupdb ')
CALL GSTATS_LABEL(1778,'IO-','READOBA grid_nearest ')
CALL GSTATS_LABEL(1783,'IO-','DB IN MKGLOBSTAB ')
CALL GSTATS_LABEL(1784,'IO-','DB IN SURAD ')
CALL GSTATS_LABEL(1785,'IO-','DB IN SUAMV ')
CALL GSTATS_LABEL(1786,'IO-','SARINVERT WRITE ')
CALL GSTATS_LABEL(1787,'IO-','ICLOSEFDB in WAM ')
CALL GSTATS_LABEL(1788,'IO-','DB IN ECSET ')
CALL GSTATS_LABEL(1789,'IO-','DB IN SCREEN ')
CALL GSTATS_LABEL(1790,'IO-','DB IN SCREEN ')
CALL GSTATS_LABEL(1791,'IO-','DB in READOBA ')
CALL GSTATS_LABEL(1792,'IO-','DB in WRITEOBA ')
CALL GSTATS_LABEL(1793,'IO-','DB in OBATABS ')
CALL GSTATS_LABEL(1796,'IO-','DB in SUREO3 ')
CALL GSTATS_LABEL(1797,'IO-','DB in SULIMB ')
CALL GSTATS_LABEL(1798,'IO-','SUJQDATA ')
CALL GSTATS_LABEL(1799,'IO-','DB in SUOBAREA ')
!   counters 1800 to 1810 :  trans package  for  serial part
CALL GSTATS_LABEL(1801,'SER','SULEG ')
CALL GSTATS_LABEL(1802,'SER','SETUP_TRANS ')
CALL GSTATS_LABEL(1803,'SER','SUTRLE ')
CALL GSTATS_LABEL(1804,'SER','DIST_SPEC_CONTROL_SERIAL')
CALL GSTATS_LABEL(1805,'SER','TRGTOL init')
CALL GSTATS_LABEL(1806,'SER','TRLTOG init')
CALL GSTATS_LABEL(1807,'SER','INV_TRANS init')
CALL GSTATS_LABEL(1808,'SER','DIR_TRANS init')
CALL GSTATS_LABEL(1809,'SER','INV_TRANSAD init')
CALL GSTATS_LABEL(1810,'SER','DIR_TRANSAD init')
CALL GSTATS_LABEL(1811,'SER','SUPHEC ')
CALL GSTATS_LABEL(1812,'SER','EVJCDFI array  ')
CALL GSTATS_LABEL(1813,'SER','SLCOMM2a ')
CALL GSTATS_LABEL(1815,'SER','SLEXTPOLAD array ')
CALL GSTATS_LABEL(1817,'SER','SUVARBC ')
CALL GSTATS_LABEL(1818,'SER','SUECRAD ')
CALL GSTATS_LABEL(1819,'SER','COMMJBDAT array ')
CALL GSTATS_LABEL(1820,'SER','COMMJBDAT zbuf ')
CALL GSTATS_LABEL(1821,'SER','COMMJBBAL zbuf ')
CALL GSTATS_LABEL(1822,'SER','COMMJBBAL array ')
CALL GSTATS_LABEL(1823,'SER','TRANSDIR_MDLAD array ')
CALL GSTATS_LABEL(1824,'SER','COMTC doloops  ')
CALL GSTATS_LABEL(1825,'SER','GATHERTC array ')
CALL GSTATS_LABEL(1826,'SER','GATHERCOSTO array  ')
CALL GSTATS_LABEL(1827,'SER','TRANSINV_MD 1')
CALL GSTATS_LABEL(1828,'SER','TRANSINV_MD 2')  
CALL GSTATS_LABEL(1829,'SER','PRE_PRSTA in SCREEN ')  
CALL GSTATS_LABEL(1830,'SER','SUSCAL ')
CALL GSTATS_LABEL(1831,'SER','COMTC array  ')
CALL GSTATS_LABEL(1832,'SER','CNT4AD OBSPREP ')
CALL GSTATS_LABEL(1833,'SER','SETUP SUSPECG ')
CALL GSTATS_LABEL(1834,'SER','SLEXTPOL1 doloop ')
CALL GSTATS_LABEL(1835,'SER','BALOMEGATL zeroing  ')
CALL GSTATS_LABEL(1836,'SER','BALOMEGATL do loop  ')
CALL GSTATS_LABEL(1837,'SER','BALOMEGATL array  ')
CALL GSTATS_LABEL(1838,'SER','CAIN ')
CALL GSTATS_LABEL(1839,'SER','GETMINI array ')
CALL GSTATS_LABEL(1840,'SER','BALOMEGATL arrays  ')
CALL GSTATS_LABEL(1841,'SER','BALOMEGAAD arrays 1 ')
CALL GSTATS_LABEL(1842,'SER','BALOMEGAAD arrays 2 ')
CALL GSTATS_LABEL(1843,'SER','WRITE_SPEC ')
CALL GSTATS_LABEL(1844,'SER','CVAR2IN ADDBGS ')
CALL GSTATS_LABEL(1845,'SER','CNT4 SIGMASTER ')
CALL GSTATS_LABEL(1846,'SER','SCATTER_CTLVEC pack')
CALL GSTATS_LABEL(1847,'SER','MULTISCATTER_CTLVEC pack')
CALL GSTATS_LABEL(1849,'SER','THIN_RED_PRESORT ')
CALL GSTATS_LABEL(1850,'SER','COPY_SPEC_2SPA arrays ')
CALL GSTATS_LABEL(1852,'SER','NEW_THINNER_NO_SQ doloop ')
CALL GSTATS_LABEL(1853,'SER','NEW_THINNER_NO_SQ KEYSORT ')
CALL GSTATS_LABEL(1856,'SER','SLINT setup ')
CALL GSTATS_LABEL(1857,'SER','GATHER_CTLVEC unpack ')
CALL GSTATS_LABEL(1858,'SER','READGRIB array ')
CALL GSTATS_LABEL(1859,'SER','MULTIGATHER_CTLVEC unpack ')
CALL GSTATS_LABEL(1860,'SER','SCAN2MTL Gaussian dist. ')
CALL GSTATS_LABEL(1861,'SER','OBSHORAD ')
CALL GSTATS_LABEL(1863,'SER','SBSFGS array ')
CALL GSTATS_LABEL(1864,'SER','GATHERTC doloop ')
CALL GSTATS_LABEL(1866,'SER','PRECH doloop')
CALL GSTATS_LABEL(1868,'SER','SUHIFCEAD')
CALL GSTATS_LABEL(1869,'SER','SLINTAD ')
CALL GSTATS_LABEL(1871,'SER','ESTSIG array ')
CALL GSTATS_LABEL(1872,'SER','TRANSDIR_MDLAD ')
CALL GSTATS_LABEL(1873,'SER','BALNONLINAD  DOLOOP')
CALL GSTATS_LABEL(1875,'SER','SUGRIDUG array ')
CALL GSTATS_LABEL(1876,'SER','BCASTCOV PACK/UNPACK ')
CALL GSTATS_LABEL(1877,'SER','MKGLOBSTAB')
CALL GSTATS_LABEL(1878,'SER','IOSTREAM GRIB_API ENC')
CALL GSTATS_LABEL(1879,'SER','IOSTREAM GRIB_API DEC')
CALL GSTATS_LABEL(1880,'SER','SUOBS STAGE 1')
CALL GSTATS_LABEL(1881,'SER','ORCVGPF doloop ')
CALL GSTATS_LABEL(1883,'SER','SARAS WAM ')
CALL GSTATS_LABEL(1884,'SER','SARINVERT WAM ')
CALL GSTATS_LABEL(1887,'SER','ESTSIG doloop ')
CALL GSTATS_LABEL(1889,'SER','stepotl user_clock ')
CALL GSTATS_LABEL(1890,'SER','BRPTOB array1 ')
CALL GSTATS_LABEL(1891,'SER','BRPTOB array2 ')
CALL GSTATS_LABEL(1901,'SER','CONSTRUCT BUTTERFLY ')

!CALL GSTATS_LABEL(1903,'SER','CNT4 IFLUSHFDB')
CALL GSTATS_LABEL(1904,'SER','UPDTIM setup ')
CALL GSTATS_LABEL(1905,'SER','UPDTIM update ')
CALL GSTATS_LABEL(1906,'SER','RADDIAG in RADDRV ')
CALL GSTATS_LABEL(1909,'SER','WAMODEL ')
CALL GSTATS_LABEL(1911,'SER','FPOSHOR doloop ')
CALL GSTATS_LABEL(1912,'SER','PPSYDH ')
CALL GSTATS_LABEL(1913,'SER','PPEDDHEC ')
CALL GSTATS_LABEL(1916,'SER','WROUTGPGB reorder  ')
CALL GSTATS_LABEL(1917,'SER','WRPLFP ')
CALL GSTATS_LABEL(1918,'SER','CONGRAD ')
CALL GSTATS_LABEL(1924,'SER','WVCOUPLE  WVALLOC ')
CALL GSTATS_LABEL(1925,'SER','GPNORM1 doloop ')
CALL GSTATS_LABEL(1928,'SER','SUJBCOR 2 ')
CALL GSTATS_LABEL(1929,'SER','SUJBCOR 3 ')
CALL GSTATS_LABEL(1930,'SER','OPDIS ')
CALL GSTATS_LABEL(1931,'SER','COMMSPNORM DOLOOP ')
CALL GSTATS_LABEL(1933,'SER','DOT_PRODUCT ')
CALL GSTATS_LABEL(1934,'SER','SU0YOMA ')
CALL GSTATS_LABEL(1935,'SER','OBADAT ')
CALL GSTATS_LABEL(1936,'SER','SUVAZX VARBC ')
CALL GSTATS_LABEL(1937,'SER','LAID*IOBS IN SLINT ')
CALL GSTATS_LABEL(1938,'SER','STORE_TRAJ_MAIN ')
CALL GSTATS_LABEL(1939,'SER','SUSC2B')
CALL GSTATS_LABEL(1940,'SER','CNT4 SMS ')
CALL GSTATS_LABEL(1941,'SER','CNT4 file')
CALL GSTATS_LABEL(1942,'SER','pack_loc_tovs ')
CALL GSTATS_LABEL(1943,'SER','GRID_BICONSERV ')
CALL GSTATS_LABEL(1944,'SER','GRID_BICUBIC ')
CALL GSTATS_LABEL(1945,'SER','GRID_BILINEAR ')
CALL GSTATS_LABEL(1946,'SER','WRITE_GRID_GRIB doloop ')
CALL GSTATS_LABEL(1948,'SER','ECSET SETTC')
CALL GSTATS_LABEL(1949,'SER','ECSET ')
CALL GSTATS_LABEL(1950,'SER','SPNORMAVE  array')
CALL GSTATS_LABEL(1952,'SER','WRMLFP do1')
CALL GSTATS_LABEL(1953,'SER','WRMLFP do2')
CALL GSTATS_LABEL(1954,'SER','SURAD SUADVAR ')
CALL GSTATS_LABEL(1956,'SER','copy_spa_spec ')
CALL GSTATS_LABEL(1957,'SER','MKGLOBSTAB KEYSORT ')
CALL GSTATS_LABEL(1958,'SER','BGVECS + Gaussian dist')
CALL GSTATS_LABEL(1959,'SER','PREPPCM ')
CALL GSTATS_LABEL(1960,'SER','DOT_PRODUCT_CTLVEC BROADCREAL ')
CALL GSTATS_LABEL(1961,'SER','EVCOST SUM ')
CALL GSTATS_LABEL(1962,'SER','DOT_PRODUCT_CTLVEC 1 ')
CALL GSTATS_LABEL(1963,'SER','DOT_PRODUCT_CTLVEC 2 ')
CALL GSTATS_LABEL(1965,'SER','DECIS IN LOOP3 ')
CALL GSTATS_LABEL(1966,'SER','DECIS DUPLI ')
CALL GSTATS_LABEL(1967,'SER','REDUN 1 ')
CALL GSTATS_LABEL(1969,'SER','REDUN 4 ')
CALL GSTATS_LABEL(1973,'SER','CNT2 ')
CALL GSTATS_LABEL(1974,'SER','SUOBS SU_ERRORS ')
CALL GSTATS_LABEL(1976,'SER','IFLUSHFDBSUBS WAM ')
CALL GSTATS_LABEL(1977,'SER','SU_GRIB_API ')
CALL GSTATS_LABEL(1978,'SER','VARBC_SETUP_TRAJ')
CALL GSTATS_LABEL(1979,'SER','VARBC_SETUP_MIN')
CALL GSTATS_LABEL(1980,'SER','STEPO ')
CALL GSTATS_LABEL(1984,'SER','GETCURR WAM ')
CALL GSTATS_LABEL(1985,'SER','in GP_MODEL_AD 7 ')
CALL GSTATS_LABEL(1987,'SER','SPEC2FDB WAM ')

!section serial but possible to put OMP
! timing < 1%
CALL GSTATS_LABEL(1892,'SER','MPEXCHNG WAM doloop1 ')
CALL GSTATS_LABEL(1893,'SER','MPEXCHNG WAM doloop2 ')
CALL GSTATS_LABEL(1894,'SER','TRANSINV_MDL')
CALL GSTATS_LABEL(1895,'SER','TRAJ_MAIN_MOD   3')
CALL GSTATS_LABEL(1896,'SER','SCAN2MTL    3')
CALL GSTATS_LABEL(1898,'SER','DIGFILAD    ')
CALL GSTATS_LABEL(1899,'SER','BALOMEGAAD do ')
CALL GSTATS_LABEL(1986,'SER','CVARGPTL ')
CALL GSTATS_LABEL(1988,'SER','GEFGER ')
CALL GSTATS_LABEL(1989,'SER','GRID_BILINEAR 2 ')
CALL GSTATS_LABEL(1991,'SER','SUJBCOSU ')
CALL GSTATS_LABEL(1992,'SER','TRANSDIR_FP ')
CALL GSTATS_LABEL(1993,'SER','WVWG2RG ')
CALL GSTATS_LABEL(1994,'SER','WVCOUPLE  3')
CALL GSTATS_LABEL(1995,'SER','ESTSIG ')
CALL GSTATS_LABEL(1996,'SER','MAKEGRID WAM ')
CALL GSTATS_LABEL(1997,'SER','WROUTSPGB ')
CALL GSTATS_LABEL(1998,'SER','SUSPECG array ')
CALL GSTATS_LABEL(1999,'SER','GOM_PRINT_RMS')

!more SER counters
CALL GSTATS_LABEL(2000,'SER','WAVXFORM ')
CALL GSTATS_LABEL(2001,'SER','SUBFPOS ')
CALL GSTATS_LABEL(2002,'SER','PPFIDH  ')
CALL GSTATS_LABEL(2003,'SER','WRPGR   ')
CALL GSTATS_LABEL(2010,'SER','MWAVE_PUT_TL')
CALL GSTATS_LABEL(2011,'SER','MWAVE_PUT_AD')
CALL GSTATS_LABEL(2012,'SER','MWAVE_NEAREST')
CALL GSTATS_LABEL(2013,'SER','MWAVE_PUT')
CALL GSTATS_LABEL(2014,'SER','MWAVE_GET')
CALL GSTATS_LABEL(2015,'SER','MWAVE_GET_TL')
CALL GSTATS_LABEL(2016,'SER','MWAVE_OBS2GP')
CALL GSTATS_LABEL(2017,'SER','MWAVE_IOBS2GP')
CALL GSTATS_LABEL(2023,'SER','GOM_MAKE_POST_INTERP')

LGSTATS_LABEL=.TRUE.

END SUBROUTINE WAM_GSTATS_SETUP_LABELS

! ------------------------------------------------------------------------------------------

SUBROUTINE WAM_GSTATS_FILE_OPEN(IUGSTATS)
USE YOMGSTATS,  ONLY : LSTATS, NPRNT_STATS
USE MPL_MODULE, ONLY : MPL_MYRANK
INTEGER, INTENT(OUT) :: IUGSTATS
CHARACTER(LEN=512) :: CGSTATS_FILE
IUGSTATS = -1
IF (LSTATS) THEN
  IF (MPL_MYRANK() == 1) THEN
    CALL GET_ENVIRONMENT_VARIABLE("GSTATS_FILE",CGSTATS_FILE)
    IF (TRIM(CGSTATS_FILE) == '') CGSTATS_FILE = "gstats"
    OPEN(NEWUNIT=IUGSTATS, FILE=TRIM(CGSTATS_FILE))
  ENDIF
ENDIF
END SUBROUTINE WAM_GSTATS_FILE_OPEN

! ------------------------------------------------------------------------------------------

SUBROUTINE WAM_GSTATS_FILE_CLOSE(IUGSTATS)
USE YOMGSTATS,  ONLY : LSTATS
INTEGER, INTENT(IN) :: IUGSTATS
IF (LSTATS) THEN
  IF (IUGSTATS /= -1) CLOSE( IUGSTATS )
ENDIF
END SUBROUTINE WAM_GSTATS_FILE_CLOSE

! ------------------------------------------------------------------------------------------

END MODULE