! (C) Copyright 1989- ECMWF.
!
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!
MODULE SINFLX_CUF_PARAMETRISE_MOD
  CONTAINS
  ATTRIBUTES(DEVICE) SUBROUTINE SINFLX_CUF_PARAMETRISE (ICALL, KIJS, KIJL, LUPDTUS, FL1, WAVNUM, CINV, XK2CG, WSWAVE, WDWAVE,  &
  & AIRD, RAORW, WSTAR, CICOVER, COSWDIF, SINWDIF2, FMEAN, HALP, FMEANWS, FLM, UFRIC, TAUW, TAUWDIR, Z0M, Z0B, CHRNCK, PHIWA,  &
  & FLD, SL, SPOS, MIJ, RHOWGDFTH, XLLWS, IJ)
    
    ! ----------------------------------------------------------------------
    
    !**** *SINFLX* - UPDATE STRESS AND COMPUTE WIND INPUT SOURCE TERM.
    
    ! ----------------------------------------------------------------------
    
    USE STRESSO_CUF_PARAMETRISE_MOD, ONLY: STRESSO_CUF_PARAMETRISE
    USE SINPUT_CUF_PARAMETRISE_MOD, ONLY: SINPUT_CUF_PARAMETRISE
    USE HALPHAP_CUF_PARAMETRISE_MOD, ONLY: HALPHAP_CUF_PARAMETRISE
    USE FRCUTINDEX_CUF_PARAMETRISE_MOD, ONLY: FRCUTINDEX_CUF_PARAMETRISE
    USE FEMEANWS_CUF_PARAMETRISE_MOD, ONLY: FEMEANWS_CUF_PARAMETRISE
    USE AIRSEA_CUF_PARAMETRISE_MOD, ONLY: AIRSEA_CUF_PARAMETRISE
    USE cudafor
    USE PARKIND_WAVE, ONLY: JWIM, JWRB, JWRU
    
    USE YOWCOUP, ONLY: LWCOU_D, LLCAPCHNK_D, LLGCBZ0_D, LLNORMAGAM_D
    USE YOWPARAM, ONLY: NANG_D, NFRE_D
    USE YOWPHYS, ONLY: DTHRN_A_D, DTHRN_U_D
    USE YOWWNDG, ONLY: ICODE_D, ICODE_CPL_D
    
    
    ! ----------------------------------------------------------------------
    
    IMPLICIT NONE
    INTEGER, PARAMETER :: NANG_LOKI_PARAM = 24
    INTEGER, PARAMETER :: NFRE_LOKI_PARAM = 36
    INTEGER(KIND=JWIM), VALUE, INTENT(IN) :: ICALL    !! CALL NUMBER.
    INTEGER(KIND=JWIM), VALUE, INTENT(IN) :: KIJS, KIJL    !! GRID POINT INDEXES.
    
    LOGICAL, VALUE, INTENT(IN) :: LUPDTUS    !! IF TRUE UFRIC AND Z0M WILL BE UPDATED (CALLING AIRSEA).
    
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL, NANG_LOKI_PARAM, NFRE_LOKI_PARAM) :: FL1    !! WAVE SPECTRUM.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL, NFRE_LOKI_PARAM) :: WAVNUM    !! WAVE NUMBER.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL, NFRE_LOKI_PARAM) :: CINV    !! INVERSE PHASE VELOCITY.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL, NFRE_LOKI_PARAM) :: XK2CG    !! (WAVNUM)**2 * GROUP SPPED.
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: WSWAVE    !! WIND SPEED IN M/S.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL) :: WDWAVE    !! WIND DIRECTION IN RADIANS IN OCEANOGRAPHIC NOTATION.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL) :: AIRD    !! AIR DENSITY (KG/M**3).
    REAL(KIND=JWRB), INTENT(IN), DEVICE :: RAORW    !! RATIO AIR DENSITY TO WATER DENSITY.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL) :: WSTAR    !! FREE CONVECTION VELOCITY SCALE (M/S)
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(KIJL) :: CICOVER    !! SEA ICE COVER.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(NANG_LOKI_PARAM) :: COSWDIF    !! COS(TH(K)-WDWAVE(IJ))
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(NANG_LOKI_PARAM) :: SINWDIF2    !! SIN(TH(K)-WDWAVE(IJ))**2
    REAL(KIND=JWRB), INTENT(IN), DEVICE :: FMEAN    !! MEAN FREQUENCY.
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE :: HALP    !! 1/2 PHILLIPS PARAMETER
    REAL(KIND=JWRB), INTENT(OUT), DEVICE :: FMEANWS    !! MEAN FREQUENCY OF THE WINDSEA.
    REAL(KIND=JWRB), INTENT(IN), DEVICE, DIMENSION(NANG_LOKI_PARAM) :: FLM    !! SPECTAL DENSITY MINIMUM VALUE
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: UFRIC    !! FRICTION VELOCITY IN M/S.
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: TAUW    !! WAVE STRESS IN (M/S)**2
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: TAUWDIR    !! WAVE STRESS DIRECTION.
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: Z0M    !! ROUGHNESS LENGTH IN M.
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: Z0B    !! BACKGROUND ROUGHNESS LENGTH.
    REAL(KIND=JWRB), INTENT(INOUT), DEVICE, DIMENSION(KIJL) :: CHRNCK    !! CHARNOCK COEFFICIENT.
    
    REAL(KIND=JWRB), INTENT(OUT), DEVICE :: PHIWA    !! ENERGY FLUX FROM WIND INTO WAVES.
    REAL(KIND=JWRB), INTENT(OUT), DEVICE, DIMENSION(NANG_LOKI_PARAM, NFRE_LOKI_PARAM) :: FLD    !! DIAGONAL MATRIX OF FUNCTIONAL DERIVATIVE.
    REAL(KIND=JWRB), INTENT(OUT), DEVICE, DIMENSION(NANG_LOKI_PARAM, NFRE_LOKI_PARAM) :: SL    !! TOTAL SOURCE FUNCTION ARRAY.
    REAL(KIND=JWRB), INTENT(OUT), DEVICE, DIMENSION(NANG_LOKI_PARAM, NFRE_LOKI_PARAM) :: SPOS    !!  POSITIVE SINPUT ONLY.
    
    INTEGER(KIND=JWIM), INTENT(OUT), DEVICE :: MIJ(KIJL)    !! LAST FREQUENCY INDEX OF THE PROGNOSTIC RANGE.
    
    REAL(KIND=JWRB), INTENT(OUT), DEVICE, DIMENSION(NFRE_LOKI_PARAM) :: RHOWGDFTH    !! WATER DENSITY * G * DF * DTHETA
    
    REAL(KIND=JWRB), INTENT(OUT), DEVICE, DIMENSION(KIJL, NANG_LOKI_PARAM, NFRE_LOKI_PARAM) :: XLLWS    !! TOTAL WINDSEA MASK FROM INPUT SOURCE TERM.
    
    
    INTEGER(KIND=JWIM) :: K
    INTEGER(KIND=JWIM), VALUE, INTENT(IN) :: IJ
    INTEGER(KIND=JWIM) :: IUSFG, ICODE_WND
    INTEGER(KIND=JWIM) :: NGST
    
    REAL(KIND=JWRB), DEVICE :: RNFAC
    
    LOGICAL :: LLPHIWA, LLSNEG
    
    ! ----------------------------------------------------------------------
    
    
    ! UPDATE UFRIC AND Z0M
    IF (ICALL == 1) THEN
      IUSFG = 0
      IF (LWCOU_D) THEN
        ICODE_WND = ICODE_CPL_D
      ELSE
        ICODE_WND = ICODE_D
      END IF
      
      LLPHIWA = .false.
      LLSNEG = .false.
    ELSE
      IUSFG = 1
      ICODE_WND = 3
      
      LLPHIWA = .true.
      LLSNEG = .true.
    END IF
    
    IF (LLNORMAGAM_D .and. LLCAPCHNK_D) THEN
      RNFAC = 1.0_JWRB + DTHRN_A_D*(1.0_JWRB + TANH(WSWAVE(IJ) - DTHRN_U_D))
    ELSE
      RNFAC = 1.0_JWRB
    END IF
    
    
    IF (LUPDTUS) THEN
      ! increase noise level in the tail
      IF (ICALL == 1) THEN
        DO K=1,NANG_D
          FL1(IJ, K, NFRE_D) = MAX(FL1(IJ, K, NFRE_D), FLM(K))
        END DO
        
        IF (LLGCBZ0_D) THEN
          CALL HALPHAP_CUF_PARAMETRISE(KIJS, KIJL, WAVNUM, COSWDIF, FL1, HALP, IJ)
        ELSE
          HALP = 0.0_JWRB
        END IF
        
      END IF
      
      CALL AIRSEA_CUF_PARAMETRISE(KIJS, KIJL, HALP, WSWAVE, WDWAVE, TAUW, TAUWDIR, RNFAC, UFRIC, Z0M, Z0B, CHRNCK, ICODE_WND,  &
      & IUSFG, IJ)
      
    END IF
    
    ! COMPUTE WIND INPUT
    !!  FLD AND SL ARE INITIALISED IN SINPUT
    
    CALL SINPUT_CUF_PARAMETRISE(ICALL, LLSNEG, KIJS, KIJL, FL1, WAVNUM, CINV, XK2CG, WDWAVE, WSWAVE, UFRIC, Z0M, COSWDIF,  &
    & SINWDIF2, RAORW, WSTAR, RNFAC, FLD, SL, SPOS, XLLWS, IJ)
    
    
    ! MEAN FREQUENCY CHARACTERISTIC FOR WIND SEA
    CALL FEMEANWS_CUF_PARAMETRISE(KIJS, KIJL, FL1, XLLWS, FMEANWS, IJ)
    
    ! COMPUTE LAST FREQUENCY INDEX OF PROGNOSTIC PART OF SPECTRUM.
    CALL FRCUTINDEX_CUF_PARAMETRISE(KIJS, KIJL, FMEAN, FMEANWS, UFRIC, CICOVER, MIJ, RHOWGDFTH, IJ)
    
    ! UPDATE TAUW
    CALL STRESSO_CUF_PARAMETRISE(KIJS, KIJL, MIJ, RHOWGDFTH, FL1, SL, SPOS, CINV, WDWAVE, UFRIC, Z0M, AIRD, RNFAC, COSWDIF,  &
    & SINWDIF2, TAUW, TAUWDIR, PHIWA, LLPHIWA, IJ)
    
    ! ----------------------------------------------------------------------
    
    
  END SUBROUTINE SINFLX_CUF_PARAMETRISE
END MODULE SINFLX_CUF_PARAMETRISE_MOD
