! (C) Copyright 1989- ECMWF.
!
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!
MODULE YOWCOUP
  
  USE CUDAFOR
  USE PARKIND_WAVE, ONLY: JWIM, JWRB, JWRU, JWRO
  
  IMPLICIT NONE
  
  !*    ** *COUPL* - PARAMETERS FOR COUPLING.
  
  LOGICAL :: LLCAPCHNK
  LOGICAL :: LLGCBZ0
  LOGICAL :: LLNORMAGAM
  LOGICAL :: LWCOU
  LOGICAL :: LWCOUSAMEGRID = .false.
  LOGICAL :: LWNEMOCOU = .false.
  LOGICAL :: LWNEMOCOUSEND
  LOGICAL :: LWNEMOTAUOC
  LOGICAL :: LWNEMOCOUSTRN = .false.
  LOGICAL :: LWNEMOCOURECV
  LOGICAL :: LWNEMOCOUCIC = .false.
  LOGICAL :: LWNEMOCOUCIT = .false.
  LOGICAL :: LWNEMOCOUCUR = .false.
  LOGICAL :: LWNEMOCOUSTK = .false.
  LOGICAL :: LWNEMOCOUDEBUG
  LOGICAL :: LWCOU2W
  LOGICAL :: LWCOURNW
  LOGICAL :: LWCOUHMF
  LOGICAL :: LWFLUX
  LOGICAL :: LWVFLX_SNL
  LOGICAL :: LWCOUNORMS
  LOGICAL :: LLNORMIFS2WAM
  LOGICAL :: LLNORMWAM2IFS
  LOGICAL :: LLNORMWAMOUT
  LOGICAL :: LLNORMWAMOUT_GLOBAL
  CHARACTER(LEN=1024) :: CNORMWAMOUT_FILE
  INTEGER :: IUWAMNORM_OUT
  LOGICAL :: LMASK_OUT_NOT_SET
  LOGICAL :: LMASK_TASK_STR
  INTEGER(KIND=JWIM) :: KCOUSTEP
  INTEGER(KIND=JWIM), ALLOCATABLE :: LFROMTASK(:), LTOTASK(:)
  INTEGER(KIND=JWIM), ALLOCATABLE :: IJFROMTASK(:), IJTOTASK(:)
  INTEGER(KIND=JWIM), ALLOCATABLE :: ISTFROMTASK(:), ISTTOTASK(:)
  INTEGER(KIND=JWIM) :: NFROMTASKS, NTOTASKS
  INTEGER(KIND=JWIM), ALLOCATABLE :: I_MASK_IN(:)
  INTEGER(KIND=JWIM) :: N_MASK_IN
  INTEGER(KIND=JWIM), ALLOCATABLE :: I_MASK_OUT(:), J_MASK_OUT(:)
  INTEGER(KIND=JWIM) :: N_MASK_OUT
  
  INTEGER(KIND=JWIM), PARAMETER :: JTOT_TAUHF = 19  ! must be odd !!!
  REAL(KIND=JWRB) :: WTAUHF(JTOT_TAUHF)
  REAL(KIND=JWRB) :: X0TAUHF
  
  INTEGER(KIND=JWIM) :: NEMONTAU
  
  INTEGER(KIND=JWIM) :: NEMOINIDATE
  INTEGER(KIND=JWIM) :: NEMOINITIME
  INTEGER(KIND=JWIM) :: NEMOITINI
  INTEGER(KIND=JWIM) :: NEMOITEND
  REAL(KIND=JWRO) :: NEMOTSTEP
  INTEGER(KIND=JWIM) :: NEMOFRCO
  INTEGER(KIND=JWIM) :: NEMONSTEP
  INTEGER(KIND=JWIM) :: NEMOCSTEP
  INTEGER(KIND=JWIM) :: NEMOWSTEP
  
  INTEGER(KIND=JWIM) :: IFSNSTEP
  REAL(KIND=JWRB) :: IFSTSTEP
  LOGICAL :: LIFS_IO_SERV_ENABLED
  
  CHARACTER(LEN=32) :: IFSCONTEXT  ! Corresponds to ALGORITHM_STATE_MOD%GET_ALGOR_TYPE()
  INTEGER(KIND=JWIM) :: IFSNUPTRA  ! Corresponds to ALGORITHM_STATE_MOD%GET_NUPTRA()
  INTEGER(KIND=JWIM) :: IFSMUPTRA  ! Corresponds to ALGORITHM_STATE_MOD%GET_MUPTRA()
  
  PROCEDURE(OUTWSPEC_IO_SERV), POINTER :: OUTWSPEC_IO_SERV_HANDLER => NULL()
  PROCEDURE(OUTINT_IO_SERV), POINTER :: OUTINT_IO_SERV_HANDLER => NULL()
  PROCEDURE(IFSTOWAM), POINTER :: IFSTOWAM_HANDLER => NULL()
  
  !*     VARIABLE.   TYPE.     PURPOSE.
  !      ---------   -------   --------
  !      *LWCOU*     LOGICAL   CONTROLS COUPLING WITH ATMOSPHERIC MODEL
  !      *LWCOUSAMEGRID        TRUE when coupled and the atmospheric grid and the wave grid is the same
  !      *LWNEMOCOUSEND* L     SENDS DATA TO THE NEMO MODEL.
  !      *LWNEMOCOUSTK LOGI.   SEND SURFACE STOKES DRIFT TO NEMO
  !      *LWNEMOCOUSTRN LOGI.  SEND ICE WAVE STRAIN TO NEMO
  !      *LWNEMOCOURECV* L     RECV DATA FROM THE NEMO MODEL.
  !      *LWNEMOCOU* LOGICAL   CONTROLS COUPLING WITH NEMO OCEAN MODEL
  !      *LWNEMOCOUDEBUG* LO   EXTRA NETCDF DEBUGGING OUTPUT FOR NEMO COUPLING
  !      *LWNEMOTAUOC* LO      USE TAUOC OR FULL TAU FOR SENDING TO NEMO
  !      *LWNEMOCOUCIC LOGI.   USE THE ICE CONCENTRATION FROM NEMO for open ocean points as determined by the lake cover (see micep)
  !      *LWNEMOCOUCIT LOGI.   USE THE ICE THICKNESS FROM NEMO (see LWNEMOCOUCIC)
  !      *LWNEMOCOUCUR LOGI.   USE THE OCEAN CURRENTS FROM NEMO for open ocean points as determined by the lake cover (see getcurr)
  !      *LWCOU2W*   LOGICAL   CONTROLS 1-WAY OR 2-WAY COUPLING TO ATMOSPHERE
  !      *LWCOURNW*  LOGICAL   WHEN COUPLED, IF TRUE THEN THE WINDS ARE THE 10m NEUTRAL WINDS RELATIVE TO SURFACE OCEAN CURRENTS
  !      *LWCOUHMF*  LOGICAL   WHEN COUPLED, IF TRUE THEN THE SEA STATE EFFECT ON THE HEAT AND MOISTURE WILL BE ACTIVATED
  !      *LWFLUX*    LOGICAL   IF TRUE FLUXES FOR OCEAN MODEL ARE PRODUCED
  !                            FOR THE IFS.
  !      *LWVFLX_SNL LOGICAL   IF TRUE FLUXES FOR OCEAN MODEL ARE PRODUCED
  !                            WITh THE NONLINEAR SOURCE TERM CONTRIBUTION
  !      *LWCOUNORMS*LOGICAL   CONTROLS COMPUTING/PRINTING OF TRUE GLOBAL NORMS OF FIELDS
  !                            FROM/TO THE ATMOS MODEL
  !      *LLNORMIFS2WAM* LOGICAL IF TRUE NORMS FOR FIELDS PASSED FROM IFS TO WAM WILL BE PRODUCED
  !      *LLNORMWAM2IFS* LOGICAL IF TRUE NORMS FOR FIELDS PASSED FROM WAM TO IFS WILL BE PRODUCED
  !      *LLNORMWAMOUT* LOGICAL IF TRUE NORMS OF SELECTED OUPTUT FIELDS WILL BE PRODUCED
  !      *LLNORMWAMOUT_GLOBAL* LOGICAL IF TRUE NORMS OF SELECTED OUPTUT FIELDS WILL BE GLOBAL (see above)
  !      *LMASK_OUT_NOT_SET    INDICATES IF THE MASK USED FOR FIELDS RETURNED
  !                            TO IFS IS UNSET OR NOT.
  !      *LMASK_TASK_STR* LOGICAL TRUE UNTIL THE TASK STRUCTURE CORRESPONDING
  !                            TO MASK_OUT IS DETERMINED.
  !      *KCOUSTEP*  INTEGER   COUPLING TIME TO THE IFS (in seconds).
  !      *LFROMTASK* INTEGER   CONTROLS WHICH WAM TASKS CONTRIBUTE TO THE FIELDS
  !                            RETURNED TO THE IFS FROM CURRENT TASK
  !                            BY SPECIFYING THE NUMBER OF SEA POINTS THAT
  !                            CONTRIBUTES TO THE FIELDS
  !      *LTOTASK*   INTEGER   CONTROLS WHICH WAM TASKS NEED TO RECEIVE
  !                            CONTRIBUTIONS TO THE FIELDS RETURNED TO IFS
  !                            FROM CURRENT WAM TASK BY SPECIFYING
  !                            THE NUMBER OF SEA POINTS THAT ARE NEEDED.
  !      *IJFROMTASK*INTEGER   GIVES THE (global) IJ INDEX FOR EACH SEA POINT
  !                            REFERRED TO BY LFROMTASK.
  !                            SIZE IS GIVEN BY NFROMTASKS
  !      *IJTOTASK*  INTEGER   GIVES THE (global) IJ INDEX FOR EACH SEA POINT
  !                            REFERRED TO BY LTOTASK
  !                            SIZE IS GIVEN BY NTOTASKS.
  !      *ISTFROMTASK*INTEGER  GIVES THE STARTNG INDEX IN IJFROMTASK FOR ALL
  !                            IJ's THAT ARE FROM EACH TASK (if any).
  !      *ISTTOTASK*  INTEGER  GIVES THE STARTNG INDEX IN IJTOTASK FOR ALL
  !                            IJ's THAT ARE TO BE SENT FROM EACh TASK (if any).
  !      *NFROMTASKS*INTEGER   SUM OF LFROMTASKS ON EACH TASK.
  !      *NTOTASKS*  INTEGER   SUM OF LTOTASKS ON EACH TASK.
  !      *I_MASK_IN* INTEGER   LIST OF I INDEX OF MASK_IN THAT ARE LOCAL TO TASK
  !      *N_MASK_IN* INTEGER   MAXIMUM SIZE OF I_MASK_IN
  !      *I_MASK_OUT*INTEGER   LIST OF I INDEX OF MASK_OUT THAT ARE LOCAL TO TASK
  !      *J_MASK_OUT*INTEGER   LIST OF J INDEX OF MASK_OUT THAT ARE LOCAL TO TASK
  !      *N_MASK_OUT*INTEGER   MAXIMUM SIZE OF I_MASK_OUT AND J_MASK_OUT
  !      *LLCAPCHNK* LOGICAL   IF TRUE CHARNOCK FOR HIGH WINDS WILL BE CAPPED (see chnkmin)
  !      *LLGCBZ0*   LOGICAL   IF TRUE USE GRAVITY-CAPILLARY MODEL FOR BACKGROUND ROUGHNESS
  !      *LNORMAGAM* LOGICAL   IF TRUE USE THE RENORMALISTION OF THE GROWTH RATE.
  
  !      *JTOT_TAUHF INTEGER   DIMENSION OF WTAUHF. IT MUST BE ODD !!!
  !      *WTAUHF*    REAL      INTEGRATION WEIGHT FOR TAU_PHI_HF
  !      *X0TAUHF*   REAL      LOWEST LIMIT FOR INTEGRATION IN TAU_PHI_HF: X0 *(G/USTAR)
  !
  
  !      FOR ACCUMULATED FIELDS PASSED TO NEMO:
  !      *NEMONTAU*  INTEGER   ACCUMULATION COUNT
  !
  !      *NEMOINIDATE* INTEGER NEMO INITIAL DATE
  !      *NEMOINITIME* INTEGER NEMO INITIAL TIME
  !      *NEMOITINI*   INTEGER NEMO INITIAL TIME STEP
  !      *NEMOITEND*   INTEGER NEMO FINAL TIME STEP
  !      *NEMOTSTEP*   REAL    NEMO TIMESTEP
  !      *NEMOFRCO*    INTEGER NEMO COUPLING FREQ IN WAM TIME STEPS
  !      *NEMONSTEP*   INTEGER NEMO COUPLING FREQ IN NEMO TIME STEPS
  !      *NEMOCSTEP*   INTEGER NEMO CURRENT TIME STEP
  !      *NEMOWSTEP*   INTEGER WAM  CURRENT TIME STEP
  
  !      *IFSTSTEP*    TIME STEP OF ATMOSPHERIC MODEL
  !      *IFSNSTEP*    CURRENT STEP OF ATMOSPHERIC MODEL
  !      *LIFS_IO_SERV_ENABLED* LOGICAL TRUE IF IFS IO SERVER ENABLED
  !
  !      IFSCONTEXT    Corresponds to ALGORITHM_STATE_MOD%GET_ALGOR_TYPE()
  !      IFSNUPTRA     Corresponds to ALGORITHM_STATE_MOD%GET_NUPTRA()
  !      IFSMUPTRA     Corresponds to ALGORITHM_STATE_MOD%GET_MUPTRA()
  !
  ! ----------------------------------------------------------------------
  
  LOGICAL, DEVICE :: LLGCBZ0_D
  LOGICAL, DEVICE :: LLNORMAGAM_D
  REAL(KIND=JWRB), DEVICE :: WTAUHF_D(JTOT_TAUHF)
  REAL(KIND=JWRB), DEVICE :: X0TAUHF_D
  LOGICAL, DEVICE :: LLCAPCHNK_D
  LOGICAL, DEVICE :: LWNEMOCOUSTRN_D = .false.
  LOGICAL, DEVICE :: LWNEMOCOUSTK_D = .false.
  LOGICAL, DEVICE :: LWNEMOCOUSEND_D
  LOGICAL, DEVICE :: LWNEMOCOU_D = .false.
  LOGICAL, DEVICE :: LWCOU_D
  LOGICAL, DEVICE :: LWNEMOTAUOC_D
  LOGICAL, DEVICE :: LWVFLX_SNL_D
  LOGICAL, DEVICE :: LWFLUX_D
  CONTAINS
  
  !----------------------------------------------------------------------------------------
  
  SUBROUTINE IFSTOWAM (BLK2LOC, NFIELDS, NGPTOTG, NCA, NRA, FIELDS, LWCUR, MASK_IN, NXS, NXE, NYS, NYE, FIELDG)
    USE PARKIND_WAVE, ONLY: JWIM, JWRB
    USE YOWDRVTYPE, ONLY: WVGRIDLOC, FORCING_FIELDS
    USE YOWGRID, ONLY: NPROMA_WAM, NCHNK
    USE YOWABORT, ONLY: WAM_ABORT
    TYPE(WVGRIDLOC), INTENT(IN) :: BLK2LOC
    INTEGER(KIND=JWIM), INTENT(IN) :: NFIELDS, NGPTOTG, NCA, NRA
    REAL(KIND=JWRB), INTENT(IN) :: FIELDS(NGPTOTG, NFIELDS)
    LOGICAL, INTENT(IN) :: LWCUR
    INTEGER(KIND=JWIM), INTENT(INOUT) :: MASK_IN(NGPTOTG)
    INTEGER(KIND=JWIM), INTENT(IN) :: NXS, NXE, NYS, NYE
    TYPE(FORCING_FIELDS), INTENT(INOUT) :: FIELDG
    
    IF (.not.ASSOCIATED(IFSTOWAM_HANDLER)) CALL WAM_ABORT('IFSTOWAM_HANDLER IS NOT INITIALIZED')
    CALL IFSTOWAM_HANDLER(BLK2LOC, NFIELDS, NGPTOTG, NCA, NRA, FIELDS, LWCUR, MASK_IN, NXS, NXE, NYS, NYE, FIELDG)
    
  END SUBROUTINE IFSTOWAM
  
  !----------------------------------------------------------------------------------------
  
  SUBROUTINE OUTWSPEC_IO_SERV (IJS, IJL, SPEC, MARSTYPE, CDATE, IFCST)
    USE PARKIND_WAVE, ONLY: JWIM, JWRB
    USE YOWPARAM, ONLY: NANG, NFRE
    USE YOWABORT, ONLY: WAM_ABORT
    IMPLICIT NONE
    INTEGER(KIND=JWIM), INTENT(IN) :: IJS, IJL
    REAL(KIND=JWRB), INTENT(IN) :: SPEC(IJS:IJL, NANG, NFRE)
    CHARACTER(LEN=2), INTENT(IN) :: MARSTYPE
    CHARACTER(LEN=14), INTENT(IN) :: CDATE
    INTEGER, INTENT(IN) :: IFCST
    
    IF (.not.ASSOCIATED(OUTWSPEC_IO_SERV_HANDLER)) CALL WAM_ABORT('OUTWSPEC_IO_SERV_HANDLER IS NOT INITIALIZED')
    CALL OUTWSPEC_IO_SERV_HANDLER(IJS, IJL, SPEC, MARSTYPE, CDATE, IFCST)
    
  END SUBROUTINE OUTWSPEC_IO_SERV
  
  !----------------------------------------------------------------------------------------
  
  SUBROUTINE OUTINT_IO_SERV (NIPRMOUT, BOUT, INFOBOUT, MARSTYPE, CDATE, IFCST)
    USE PARKIND_WAVE, ONLY: JWIM, JWRB
    USE YOWGRID, ONLY: NPROMA_WAM, NCHNK
    USE YOWABORT, ONLY: WAM_ABORT
    IMPLICIT NONE
    INTEGER(KIND=JWIM), INTENT(IN) :: NIPRMOUT
    REAL(KIND=JWRB), INTENT(IN) :: BOUT(NPROMA_WAM, NIPRMOUT, NCHNK)
    INTEGER(KIND=JWIM), INTENT(IN) :: INFOBOUT(NIPRMOUT, 3)
    CHARACTER(LEN=2), INTENT(IN) :: MARSTYPE
    CHARACTER(LEN=14), INTENT(IN) :: CDATE
    INTEGER(KIND=JWIM), INTENT(IN) :: IFCST
    
    IF (.not.ASSOCIATED(OUTINT_IO_SERV_HANDLER)) CALL WAM_ABORT('OUTINT_IO_SERV_HANDLER IS NOT INITIALIZED')
    CALL OUTINT_IO_SERV_HANDLER(NIPRMOUT, BOUT, INFOBOUT, MARSTYPE, CDATE, IFCST)
    
  END SUBROUTINE OUTINT_IO_SERV
  
  !----------------------------------------------------------------------------------------
  
END MODULE YOWCOUP
