! (C) Copyright 1989- ECMWF.
!
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!
MODULE YOWPCONS
  
  USE CUDAFOR
  USE PARKIND_WAVE, ONLY: JWIM, JWRB, JWRU
  
  IMPLICIT NONE
  
  !*    *PARAMETER* OF GLOBAL CONSTANTS.
  !     THE NON PARAMETRIC CONSTANTS WILL BE RESET BY CALLING INIWCST !!!!
  
  REAL(KIND=JWRB) :: G = 9.806_JWRB
  REAL(KIND=JWRB) :: GM1 = 0.101978381_JWRB
  REAL(KIND=JWRB), PARAMETER :: OLDPI = 3.1415927_JWRB
  REAL(KIND=JWRB) :: PI = OLDPI
  REAL(KIND=JWRB), PARAMETER :: CIRC = 40000000.0_JWRB
  REAL(KIND=JWRB) :: ZPI = 6.2831854_JWRB
  REAL(KIND=JWRB) :: THREEZPI = 18.849555922_JWRB
  REAL(KIND=JWRB) :: ZPI4GM1 = 158.93794172_JWRB
  REAL(KIND=JWRB) :: ZPI4GM2 = 16.208233910_JWRB
  REAL(KIND=JWRB) :: ZPISQRT = 1.7724539_JWRB
  REAL(KIND=JWRB) :: ZCONST = 0.0281349_JWRB
  REAL(KIND=JWRB) :: RAD = 0.017453293_JWRB
  REAL(KIND=JWRB) :: DEG = 57.295778667_JWRB
  REAL(KIND=JWRB) :: R = 6366198.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: EPSMIN = 0.1E-32_JWRB
  REAL(KIND=JWRB), PARAMETER :: DKMAX = 40.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: TAUOCMIN = 0.01_JWRB
  REAL(KIND=JWRB), PARAMETER :: TAUOCMAX = 50.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: PHIEPSMIN = -3276.80_JWRB
  REAL(KIND=JWRB), PARAMETER :: PHIEPSMAX = -0.05_JWRB
  REAL(KIND=JWRB), PARAMETER :: WSEMEAN_MIN = 0.001_JWRB
  REAL(KIND=JWRB) :: ZMISS = -999.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: ROAIR = 1.225_JWRB
  REAL(KIND=JWRB), PARAMETER :: ROWATER = 1000.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: ROWATERM1 = 1.0_JWRB / ROWATER
  REAL(KIND=JWRB), PARAMETER :: YEPS = ROAIR / ROWATER
  REAL(KIND=JWRB), PARAMETER :: YINVEPS = 1.0_JWRB / YEPS
  REAL(KIND=JWRB), PARAMETER :: GAM_SURF = 0.0717_JWRB  !!!! will need to be adapted if you change ROWATER
  REAL(KIND=JWRB), PARAMETER :: SURFT = GAM_SURF / ROWATER
  REAL(KIND=JWRB) :: SQRTGOSURFT  !! SQRT(G/SURFT) (see wavemdl)
  REAL(KIND=JWRB), PARAMETER :: WSTAR0 = 0.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: Rconstant = 287.16_JWRB  ! The gas constant
  REAL(KIND=JWRB), PARAMETER :: EpsWaterVapor = 0.61_JWRB  ! The mass ratio of  water vapor to dry air
  REAL(KIND=JWRB), PARAMETER :: EPSUS = 1.0E-6_JWRB
  REAL(KIND=JWRB), PARAMETER :: EPSU10 = SQRT(1.0E-3_JWRB)
  
  REAL(KIND=JWRB), PARAMETER :: ACD = 8.0E-4_JWRB
  REAL(KIND=JWRB), PARAMETER :: BCD = 8.0E-5_JWRB
  
  REAL(KIND=JWRB), PARAMETER :: ACDLIN = 0.0008_JWRB
  REAL(KIND=JWRB), PARAMETER :: BCDLIN = 0.00047_JWRB
  
  REAL(KIND=JWRB), PARAMETER :: C1CD = 1.03E-3_JWRB
  REAL(KIND=JWRB), PARAMETER :: C2CD = 0.04E-3_JWRB
  REAL(KIND=JWRB), PARAMETER :: P1CD = 1.48_JWRB
  REAL(KIND=JWRB), PARAMETER :: P2CD = -0.21_JWRB
  
  REAL(KIND=JWRB), PARAMETER :: CDMAX = 0.0025_JWRB
  
  REAL(KIND=JWRB), PARAMETER :: FM2FP = 0.9_JWRB
  
  !*    VARIABLE.   TYPE.     PURPOSE.
  !     ---------   -------   --------
  !     *G*         REAL      ACCELLERATION OF GRAVITY.
  !     *GM1*       REAL      1/G.
  !     *OLDPI*     REAL      OLD VALUE USED FOR PI (PRIOR TO CY21R3).
  !     *PI*        REAL      PI (See SUB. INIWCST).
  !     *CIRC*      REAL      EARTH CIRCUMFERENCE (METRES).
  !     *RAD*       REAL      PI / 180.
  !     *DEG*       REAL      180. / PI.
  !     *ZPI*       REAL      2. * PI.
  !     *ZPI4GM1*   REAL      ZPI**4/G*
  !     *ZPI4GM2*   REAL      ZPI**4/G**2
  !     *ZPISQRT*   REAL      SQRT(PI)
  !     *ZCONST*    REAL      1./(8.*PI*SQRT(2.))
  !     *R*         REAL      EARTH RADIUS        (METRES).
  !     *EPSMIN*    REAL      SMALL NUMBER
  !     *DKMAX*     REAL      MAXIMUM VALUE OF DEPTH*WAVENUMBER BEFORE
  !                           SIMPLIFYING TO DEEP WATER EXPRESIONS.
  !     *BF2MAX*    REAL      MAXIMUM VALUE ALLOWED FOR BFI SQUARED
  !     *BF2MIN*    REAL      MINIMUM VALUE ALLOWED FOR BFI SQUARED
  !     *C4MAX*     REAL      MAXIMUM VALUE ALLOWED FOR KURTOSIS
  !     *C4MIN*     REAL      MINIMUM VALUE ALLOWED FOR KUTOSIS
  !     *PHIEPSMIN* REAL      MINIMUM VALUE ALLOWED FOR NORMALISED ENERGY FLUX INTO OCEAN
  !     *PHIEPSMAX* REAL      MAXIMUM VALUE ALLOWED FOR NORMALISED ENERGY FLUX INTO OCEAN
  !     *WSEMEAN_MIN* REAL    MINIMUM VALUE ALLOWED FOR WSEMEAN
  !     *ZMISS*     REAL      MISSING DATA INDICATOR
  !                          (SET IN CHIEF OR VIA THE IFS).
  !     *ROAIR*     REAL      AIR DENSITY.
  !     *ROWATER*   REAL      WATER DENSITY.
  !     *ROWATERM1* REAL      1 /(WATER DENSITY).
  !     *GAM_SURF*  REAL      SURFACE TENSION (in N/m)
  !     *SURFT*     REAL      SURFACE TENSION divided by water density (in m**3 s**-2)
  !     *YEPS*      REAL      ROAIR/ROWATER.
  !     *YINVEPS*   REAL      1./YEPS.
  !     *WSTAR0*    REAL      DEFAULT VALUE FOR w*.
  !     *EPSUS*     REAL      SMALL NUMBER ADDED TO THE SQUARE OF USTAR
  !                           WHEN COMPUTING CD OR THE CHARNOCK COEF.
  !     *EPSU10*    REAL      SMALL NUMBER TO INSURE U10 IS NEVER 0
  
  !     *ACD*       REAL      COEFFICIENTS FOR SIMPLE CD(U10) RELATION
  !     *BCD*       REAL      CD = ACD + BCD*U10
  
  !     *ACDLIN*    REAL      COEFFICIENTS FOR SIMPLE LINEARISED CD(U10,CHARNOCK) RELATION
  !     *BCDLIN*    REAL      CD = ACDLIN + BCDLIN*SQRT(CHARNOCK) * U10
  
  !     USE HERSBACH 2011 FOR CD(U10) (SEE ALSO EDSON et al. 2013)
  !     C_D = (C1CD + C2CD*U10**P1CD)*U10**P2CD
  
  !     *CDMAX*    REAL      MAXIMUM CD ALLOWED FOR ALL CD(U10) RELATIONS
  
  !     *FM2FP*    REAL      EMPIRICAL FACTOR FOR THE CONVERSION OF WINDSEA MEAN FREQUENCY
  !                          TO WINDSEA PEAK FREQUENCY.
  !----------------------------------------------------------------------
  
  REAL(KIND=JWRB), DEVICE :: SQRTGOSURFT_D
  REAL(KIND=JWRB), DEVICE :: G_D = 9.806_JWRB
  REAL(KIND=JWRB), DEVICE :: ZPI4GM2_D = 16.208233910_JWRB
  REAL(KIND=JWRB), DEVICE :: ZPI4GM1_D = 158.93794172_JWRB
  REAL(KIND=JWRB), DEVICE :: ZPI_D = 6.2831854_JWRB
  REAL(KIND=JWRB), DEVICE :: GM1_D = 0.101978381_JWRB
END MODULE YOWPCONS
