! (C) Copyright 1989- ECMWF.
!
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
! In applying this licence, ECMWF does not waive the privileges and immunities
! granted to it by virtue of its status as an intergovernmental organisation
! nor does it submit to any jurisdiction.
!
MODULE YOWWIND
  
  USE CUDAFOR
  USE PARKIND_WAVE, ONLY: JWIM, JWRB, JWRU
  
  USE YOWDRVTYPE, ONLY: FORCING_FIELDS
  
  IMPLICIT NONE
  
  REAL(KIND=JWRB) :: WSPMIN
  REAL(KIND=JWRB), PARAMETER :: WSPMIN_RESET_TAUW = 4.0_JWRB
  REAL(KIND=JWRB), PARAMETER :: USTMIN_RESET_TAUW = 0.08_JWRB
  REAL(KIND=JWRB), PARAMETER :: RWFAC = 0.5_JWRB
  
  CHARACTER(LEN=14) :: CDATEWL
  CHARACTER(LEN=14) :: CDAWIFL
  CHARACTER(LEN=14) :: CDATEWO
  CHARACTER(LEN=14) :: CDATEFL
  CHARACTER(LEN=14) :: CDTNEXT
  CHARACTER(LEN=80) :: CWDFILE
  
  INTEGER(KIND=JWIM) :: IIG
  INTEGER(KIND=JWIM) :: NC
  INTEGER(KIND=JWIM) :: NR
  INTEGER(KIND=JWIM) :: NXFFS
  INTEGER(KIND=JWIM) :: NXFFE
  INTEGER(KIND=JWIM) :: NYFFS
  INTEGER(KIND=JWIM) :: NYFFE
  INTEGER(KIND=JWIM) :: NXFFS_LOC
  INTEGER(KIND=JWIM) :: NXFFE_LOC
  INTEGER(KIND=JWIM) :: NYFFS_LOC
  INTEGER(KIND=JWIM) :: NYFFE_LOC
  INTEGER(KIND=JWIM) :: NSTORE
  INTEGER(KIND=JWIM) :: IUNITW = 0
  INTEGER(KIND=JWIM) :: NBITW = 8100000
  
  TYPE(FORCING_FIELDS) :: FF_NEXT
  
  LOGICAL :: LLWSWAVE
  LOGICAL :: LLWDWAVE
  LOGICAL :: LLNEWCURR
  
  !*     VARIABLE.   TYPE.     PURPOSE.
  !      ---------   -------   --------
  !      *WSPMIN*    REAL      MINIMUM WIND SPEED ALLOWED IN WAM (SEE USERIN).
  !      *WSPMIN_RESET_TAUW    MINIMUM WIND SPEED TO RESET WAVE INDUCED STRESS
  !      *USTMIN_RESET_TAUW    MINIMUM FRICTION VELOCITY TO RESET WAVE INDUCED STRESS
  !      *RWFAC*     REAL      REDUCTION FACTOR OF THE SURFACE CURRENTS
  !                            WHEN USED TO COMPUTE THE RELATIVE WINDS
  !                            0 <= RWFAC <= 1
  !      *CDATEWL*   CHAR*14   DATE OF LAST WINDFIELD READ IN.
  !      *CDATEWO*   CHAR*14   DATE OF NEXT WIND FIELD TO BE READ.
  !      *CDAWIFL*   CHAR*14   DATE OF NEXT WIND FILE NAME.
  !      *CDATEFL*   CHAR*14   DATE OF NEXT WIND FILE TO BE ACCESSED.
  !      *CDTNEXT*   CHAR*14   DATE CORRESPONDING TO TEMPORARY STORAGE ??? obsolete ???
  !                            (SEE BELOW).
  !      *CWDFILE*   CHAR*80   FILENAME FOR FILE CONTAINING WIND SPEED
  !                            AND DIRECTION TIMESERIES WHEN USED INSTEAD
  !                            OF FORCING FROM INPUT WIND FIELDS
  !                            (see USERIN).
  !      *IIG*       INTEGER   BLOCK NUMBER OF LAST WIND FIELD READ IN
  !      *NC*        INTEGER   NUMBER OF COLUMNS IN INPUT WIND ARRAY
  !      *NR*        INTEGER   NUMBER OF ROWS    IN INPUT WIND ARRAY
  !      *NXFFS      INTEGER   FIRST ROW IN FORCING_FIELDS DATA
  !      *NXFFE      INTEGER   LAST ROW  IN FORCING_FIELDS DATA
  !      *NYFFS      INTEGER   FIRST COLUMN IN FORCING_FIELDS DATA
  !      *NYFFE      INTEGER   LAST COLUMN IN FORCING_FIELDS DATA
  !      *NXFFS_LOC  INTEGER   FIRST ROW IN FORCING_FIELDS DATA FOR POINTS THAT ARE LOVAT TO A GIVEN MPI TASK
  !      *NXFFE_LOC  INTEGER   LAST ROW  IN FORCING_FIELDS DATA FOR POINTS THAT ARE LOVAT TO A GIVEN MPI TASK
  !      *NYFFS_LOC  INTEGER   FIRST COLUMN IN FORCING_FIELDS DATA FOR POINTS THAT ARE LOVAT TO A GIVEN MPI TASK
  !      *NYFFE_LOC  INTEGER   LAST COLUMN IN FORCING_FIELDS DATA FOR POINTS THAT ARE LOVAT TO A GIVEN MPI TASK
  !      *IUNITW*    INTEGER   UNIT USED TO OPEN THE INPUT WIND DATA
  !                            OR THE FILE HANDLE IF GRIBAPI IS USED.
  !      *NBITW*     INTEGER   SIZE OF DECODING BUFFER FOR FORCING DATA.
  !      *FF_NEXT*   REAL      UPDATE FOR FORCING
  
  !      *LLWSWAVE*  LOGICAL   TRUE IF PARAMETER 245 IS PART OF THE INPUT
  !      *LLWDWAVE*  LOGICAL   TRUE IF PARAMETER 249 IS PART OF THE INPUT
  !      *LLNEWCURR* LOGICAL   TRUE IF NEW CURRENTS ARE PASSED FROM IFS
  !                            TO WAM
  ! ----------------------------------------------------------------------
  
  REAL(KIND=JWRB), DEVICE :: WSPMIN_D
END MODULE YOWWIND
