# Copyright 2013-2022 Lawrence Livermore National Security, LLC and other
# Spack Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from spack.package import *
import os

class Multio(CMakePackage):
    """MultIO is a runtime-configurable multiplexer for Weather Model output of GRIB data"""

    homepage = "https://github.com/ecmwf/multio"
    # url = "file://{}/multio-1.7.0.tar.gz".format(os.getcwd())
    # manual_download = True
    url = "https://github.com/ecmwf/multio/archive/refs/tags/1.8.0.tar.gz"
    git = "https://github.com/ecmwf/multio.git"
    version("1.8.0", tag="1.8.0")
    version("1.7.0", sha256= "fc2638512f7326566bea2280fe4601c24b4df3d7cdf9787c8d20b1aebfcc971e") 

    variant("fdb", default=True, description="Save data to FDB5")
    variant("fortran", default=True, description="Build FORTRAN API and examples")
    variant("server", default="ON", description="Build with I/O-server support")

    # build dependencies
    depends_on("mpi", type='build')
    depends_on("ecbuild", type='build')
    # runtime dependencies
    depends_on("eccodes")
    depends_on("eckit")
    depends_on("metkit")
    # optional dependencies
    depends_on("fdb", when='fdb')
    depends_on("fckit", when='fortran')

    def cmake_args(self):
        args = [
                self.define_from_variant("ENABLE_MULTIO_SERVER", "server"),
                ]
        # args.append("-DENABLE_FDB_BUILD_TOOLS=OFF")
        # args.append("-DENABLE_FDB_REMOTE=OFF")
        return args
