#!/bin/bash


# Loading necessary modules
module load profile/advanced
module load autoload singularity/3.8.0--bind--openmpi--4.1.1
module load cuda/11.5.0


script_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd -P)"
base_path=$(cd "$script_dir/.." && pwd -P)

CONTAINER="${1:-$base_path/images/container_ecwam.sif}"
export CONTAINER_PATH=$(realpath $CONTAINER)


# Setting variables to pass to ecWAM launch scripts
export ECWAM_PROJECT_NAME="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam"
           export PREPROC="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-preproc"
            export PRESET="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-preset"
             export CHIEF="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-chief" 
             export MODEL="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-chief"

# Folders for runtime writing
export ECWAM_RUN_DIR=$base_path/bind_folders/run_dir
export ECWAM_DATA_DIR=$base_path/bind_folders/data_dir
export ECWAM_CACHE_PATH=$base_path/bind_folders/cache_dir


# Setting bindings
GWORK_BIND=/g100_work
GSCRATCH=/scratch_local,/g100_scratch,$TMPDIR
RUNDIRS_BIND=$ECWAM_CACHE_PATH:/cache_dir,$ECWAM_DATA_DIR:/data_dir,$ECWAM_RUN_DIR:/run_dir
# Binding the PMIX libraries for GALILEO100
PMIX_BIND=/usr/lib64/libcrypto.so.1.1,/usr/lib64/libmca_common_dstore.so.1,/usr/lib64/libmunge.so.2,/usr/lib64/pmix/lib,/usr/lib64/pmix/mca_bfrops_v20.so,/usr/lib64/pmix/mca_gds_hash.so,/usr/lib64/pmix/mca_psec_none.so,/usr/lib64/pmix/mca_psensor_heartbeat.so,/usr/lib64/pmix/mca_ptl_tcp.so,/usr/lib64/pmix/mca_bfrops_v12.so,/usr/lib64/pmix/mca_bfrops_v21.so,/usr/lib64/pmix/mca_psec_munge.so,/usr/lib64/pmix/mca_gds_ds21.so,/usr/lib64/pmix/mca_preg_native.so,/usr/lib64/pmix/mca_psec_native.so,/usr/lib64/pmix/mca_psensor_file.so,/usr/lib64/pmix/mca_pshmem_mmap.so,/usr/lib64/pmix/mca_ptl_usock.so,/opt/slurm,/lib64/libpmix.so.2,/g100,/g100_work,/usr/lib64/liblustreapi.so.1,/usr/lib64/libkeyutils.so.1,/var/spool/slurmd/conf-cache
# Singularity bind variable
export SINGULARITY_BIND=$RUNDIRS_BIND,$PMIX_BIND,$GWORK_BIND,$GSCRATCH


# Taking care of PMIX warnings
export PMIX_MCA_gds=hash



