#!/bin/bash


# Loading necessary modules
module load openmpi/4.1.4--nvhpc--23.1-cuda-11.8
module load cuda/11.8


script_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd -P)"
base_path=$(cd "$script_dir/.." && pwd -P)

CONTAINER="${1:-$base_path/images/container_ecwam.sif}"
export CONTAINER_PATH=$(realpath $CONTAINER)


# Setting variables to pass to ecWAM launch scripts
export ECWAM_PROJECT_NAME="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam"
           export PREPROC="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-preproc"
            export PRESET="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-preset"
             export CHIEF="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-chief" 
             export MODEL="singularity exec $CONTAINER_PATH /opt/ecwam/install/bin/ecwam-chief"

# Folders for runtime writing
export ECWAM_RUN_DIR=$base_path/bind_folders/run_dir
export ECWAM_DATA_DIR=$base_path/bind_folders/data_dir
export ECWAM_CACHE_PATH=$base_path/bind_folders/cache_dir


# Setting bindings
LWORK_BIND=/leonardo_work
RUNDIRS_BIND=$ECWAM_CACHE_PATH:/cache_dir,$ECWAM_DATA_DIR:/data_dir,$ECWAM_RUN_DIR:/run_dir
# Binding the PMIX libraries for LEONARDO
PMIX_BIND=/usr/lib64/liblustreapi.so.1,/usr/lib64/libkeyutils.so.1,/usr/lib64/libmunge.so.2,/opt/pmix/3.1.5
# Singularity bind variable, on Leonardo nodes there is no /scratch=$TMPDIR, but it must be binded on login nodes
export SINGULARITY_BIND=$RUNDIRS_BIND,$PMIX_BIND,$LWORK_BIND #,$TMPDIR



# Taking care of PMIX warnings
export PMIX_MCA_gds=hash
