### Built of the container

The folder contains two definition files, `singularity_ecwam.def` and `singularity_ecwam_multi.def`, which can be templated to a Singularity definition via the `template.sh` script. Executing

    ./template.sh

will output, as default, a *multistage* singularity definition file for building a container with the `nvhpc@23.1` compiler and Cuda version `cuda@11.8`. The final container image will be a `runtime` flavour, weighing around `2GB`, and holding the executables and the necessary dependencies to run them only.


The script accepts flags to change the default behaviour, namely:
+ `--nvhpc=NVHPC_VERSION`, used to pass the desired version of `nvhpc`. It can take values `23.1` and `23.9`
+ `--cuda=CUDA_VERSION`, used to pass the desired version of `cuda`. It can take values subject to `NVHPC_VERSION`, i.e.
    + for `NVHPC_VERSION=23.1` allowed values are `11.0`,`11.8`,`12.0`
    + for `NVHPC_VERSION=23.9` allowed values are `11.0`,`11.8`,`12.2`
+ `--multi=MULTI`, it accepts values `1/0`,`yes/no`,`true/false`. The default behaviour is `MULTI=false`. If `MULTI=true` is set, a *single stage* Singularity definition file for a **multi** Cuda distribution will be created. The final container image will be a `develop` flavour, weighing around `12GB`, holding the whole software stack used to compile the executables, the source codes, and the executables for all the three versions of Cuda associated with the passed `NVHPC_VERSION`
As an example, one can run the following:

        ./template.sh --nvhpc=23.9 --cuda=12.2
        ./template.sh --multi=yes --nvhpc=23.1


The `singularity_ecwam.def` file includes four main phases:
+ *Bootstrap build stage:* we bootstrap from an [nvidia container](https://catalog.ngc.nvidia.com/orgs/nvidia/containers/nvhpc) specific to the chosen versions of `nvhpc` and `cuda`, namely `nvcr.io/nvidia/nvhpc:[23.1|23.9]-devel-cuda_multi-ubuntu22.04`, which comes provided with the `nvhpc@[23.1|23.9]` and `gcc@[11.3|11.4]` compilers, as well as `openmpi@4.1.5` and the `CUDA Toolkit@[{11.0,11.8,12.0} | {11.0,11.8,12.2}]`

+ In the `%files` section we add [Spack](https://github.com/spack/spack) and the `.yaml` files to handle the spack installations. We also copy the spack recipes and mirrors of the *ECMWF* provided packages, the model itself with its source files (i.e. `/ecwam_bundle`), as well as some scripts to ease the build (see `build_files/scripts_for_build`).

+ In the `%post` section, we first install a pletora of base packages via `apt`. Hence, we proceed with the installation of *ecWAM* related packages via Spack (`concretize` and `install`). We install manually `Loki` and `FieldApi`, two packages we will eventually need to run with `OpenACC`. Hence, we compile the hybrid `MPI+OpenMP` version of the code via the script `MPI_build.sh`, as well as the `OpenACC` GPU supported one via the `openACC_build_CUDA_VERSION.sh` script.

+ *Bootstrap runtime stage:* we bootstrap from a **runtime** nvidia container specific to the chosen versions of `nvhpc` and `cuda`, namely `nvcr.io/nvidia/nvhpc:NVHPC_VERSION-runtime-cuda_CUDA_VERSION-ubuntu22.04`. We copy from the previous stage the executable and the needed libraries